@@ -30,6 +30,20 @@ public:
 };
 
 
+template<typename Arg, typename Func>
+class TemplateOutputFunction : public OutputFunction<Arg>
+{
+public:
+	TemplateOutputFunction(Func const& f) : m_func(f) {}
+	
+	virtual void operator()(Arg const& arg) {
+		m_func(arg);
+	}
+private:
+	Func m_func;
+};
+
+
 #define DECLARE_INSERTER(Class, Arg, Container, Action) \
 class Class : public OutputFunction<Arg>  \
 { \
