@@ -42,12 +42,9 @@ public:
 	
 	void ref() const { m_refCounter.ref(); }
 	
-	bool unref() const {
-		if (m_refCounter.unref()) {
-			return true;
-		} else {
+	void unref() const {
+		if (!m_refCounter.unref()) {
 			delete this;
-			return false;
 		}
 	}
 private:
