@@ -103,46 +103,69 @@ public:
 private:
 	enum { STACK_SIZE = 15*1024 };
 	enum { BRANCH_LIMIT = 20000 };
+	enum Access { RO, RW };
 	
 	static Context* getEnvContext(JSContext* cx);
 	
-	static void errorReporter(JSContext *cx, const char *msg, JSErrorReport *report);
+	static void errorReporter(
+		JSContext *cx, const char *msg, JSErrorReport *report);
 	
 	static JSBool branchCallback(JSContext *cx, JSScript *script);
 	
-	static JSBool cookieGetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+	static JSBool cookieGetter(
+		JSContext *cx, JSObject *obj, jsval id, jsval *vp);
 	
-	static JSBool cookieSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+	static JSBool cookieSetter(
+		JSContext *cx, JSObject *obj, jsval id, jsval *vp);
 	
-	static JSBool onloadSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+	static JSBool onloadSetter(
+		JSContext *cx, JSObject *obj, jsval id, jsval *vp);
 	
-	static JSBool innerHtmlSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+	static JSBool innerHtmlSetter(
+		JSContext *cx, JSObject *obj, jsval id, jsval *vp);
 	
-	static JSBool locationSetter(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+	static JSBool locationSetter(
+		JSContext *cx, JSObject *obj, jsval id, jsval *vp);
 	
-	static JSBool locationToString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool locationToString(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool windowOpen(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool windowOpen(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool javaEnabled(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool javaEnabled(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool cookieEnabled(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool alert(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool alert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool documentWrite(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool documentWrite(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool documentWriteln(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool documentWriteln(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool secondaryDocumentWrite(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool processDocumentWrite(JSContext* cx, uintN argc, jsval* argv, jsval* rval, bool newline);
+	static JSBool secondaryDocumentWriteln(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool documentGetElementById(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool processDocumentWrite(
+		JSContext* cx, uintN argc, jsval* argv, jsval* rval,
+		Origin origin, bool newline);
 	
-	static JSBool documentGetElementsByTagName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool documentGetElementById(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool imageConstructor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool documentGetElementsByTagName(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
-	static JSBool doNothing(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	static JSBool imageConstructor(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+	
+	static JSBool doNothing(
+		JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
 	
 	void setJsVersion(char const* version=0);
 	
@@ -161,17 +184,19 @@ private:
 	void jsDefineProperty(JSObject *obj, char const* name, jsval value,
 		JSPropertyOp getter, JSPropertyOp setter, uintN flags);
 	
-	void jsDefineProperty(JSObject* obj, char const* name, jsval value);
+	void jsDefineProperty(JSObject* obj, char const* name, jsval value, Access access);
 	
 	JSFunction* jsDefineFunction(JSObject *obj, char const* name,
 		JSNative call, uintN nargs, uintN flags);
 	
-	JSFunction* jsDefineFunction(JSObject *obj, char const* name, JSNative call, uintN nargs);
+	JSFunction* jsDefineFunction(
+		JSObject *obj, char const* name, JSNative call, uintN nargs, Access access);
 	
 	JSObject* jsDefineObject(JSObject *obj, char const *name,
 		JSClass *clasp, JSObject *proto, uintN flags);
 	
-	JSObject* jsDefineObject(JSObject *obj, char const *name, JSClass *clasp);
+	JSObject* jsDefineObject(
+		JSObject *obj, char const *name, JSClass *clasp, Access access);
 	
 	void jsSetElement(JSObject *obj, jsint index, jsval *vp);
 	
@@ -285,33 +310,35 @@ JSClass JsEnvironment::Context::m_sPluginClass = {
 
 inline void
 JsEnvironment::Context::jsDefineProperty(
-	JSObject* obj, char const* name, jsval value)
+	JSObject* obj, char const* name, jsval value, Access access)
 {
-	jsDefineProperty(
-		obj, name, value, 0, 0,
-		JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT
-	);
+	uintN flags = JSPROP_ENUMERATE|JSPROP_PERMANENT;
+	if (access == RO) {
+		flags |= JSPROP_READONLY;
+	}
+	jsDefineProperty(obj, name, value, 0, 0, flags);
 }
 
 inline JSFunction*
 JsEnvironment::Context::jsDefineFunction(
-	JSObject *obj, char const* name, JSNative call, uintN nargs)
+	JSObject *obj, char const* name, JSNative call, uintN nargs, Access access)
 {
-	return jsDefineFunction(
-		obj, name, call, nargs,
-		JSPROP_ENUMERATE|JSPROP_PERMANENT
-	);
+	uintN flags = JSPROP_ENUMERATE|JSPROP_PERMANENT;
+	if (access == RO) {
+		flags |= JSPROP_READONLY;
+	}
+	return jsDefineFunction(obj, name, call, nargs, flags);
 }
 
 inline JSObject*
 JsEnvironment::Context::jsDefineObject(
-	JSObject *obj, char const *name, JSClass *clasp)
+	JSObject *obj, char const *name, JSClass *clasp, Access access)
 {
-	return jsDefineObject(
-		obj, name, clasp, 0,
-		JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT
-	);
-
+	uintN flags = JSPROP_ENUMERATE|JSPROP_PERMANENT;
+	if (access == RO) {
+		flags |= JSPROP_READONLY;
+	}
+	return jsDefineObject(obj, name, clasp, 0, access);
 }
 
 
@@ -334,22 +361,22 @@ JsEnvironment::Context::Context(URI const& page_url)
 		throw JsException();
 	}
 	
-	jsDefineProperty(m_pWindowObj, "self", OBJECT_TO_JSVAL(m_pWindowObj));
-	jsDefineProperty(m_pWindowObj, "window", OBJECT_TO_JSVAL(m_pWindowObj));
-	jsDefineProperty(m_pWindowObj, "top", OBJECT_TO_JSVAL(m_pWindowObj));
-	jsDefineProperty(m_pWindowObj, "parent", OBJECT_TO_JSVAL(m_pWindowObj));
-	jsDefineProperty(m_pWindowObj, "history", OBJECT_TO_JSVAL(jsNewArrayObject(0, 0)));
-	jsDefineFunction(m_pWindowObj, "alert", &alert, 1);
-	jsDefineFunction(m_pWindowObj, "open", &windowOpen, 1);
-	jsDefineFunction(m_pWindowObj, "setTimeout", &doNothing, 2);
-	jsDefineFunction(m_pWindowObj, "setInterval", &doNothing, 2);
-	jsDefineFunction(m_pWindowObj, "clearTimeout", &doNothing, 1);
-	jsDefineFunction(m_pWindowObj, "clearInterval", &doNothing, 1);
-	jsDefineFunction(m_pWindowObj, "focus", &doNothing, 0);
-	jsDefineFunction(m_pWindowObj, "blur", &doNothing, 0);
-	jsDefineFunction(m_pWindowObj, "resizeTo", &doNothing, 2);
-	jsDefineFunction(m_pWindowObj, "moveTo", &doNothing, 2);
-	jsDefineFunction(m_pWindowObj, "Image", &imageConstructor, 0);
+	jsDefineProperty(m_pWindowObj, "self", OBJECT_TO_JSVAL(m_pWindowObj), RW);
+	jsDefineProperty(m_pWindowObj, "window", OBJECT_TO_JSVAL(m_pWindowObj), RO);
+	jsDefineProperty(m_pWindowObj, "top", OBJECT_TO_JSVAL(m_pWindowObj), RW);
+	jsDefineProperty(m_pWindowObj, "parent", OBJECT_TO_JSVAL(m_pWindowObj), RW);
+	jsDefineProperty(m_pWindowObj, "history", OBJECT_TO_JSVAL(jsNewArrayObject(0, 0)), RO);
+	jsDefineFunction(m_pWindowObj, "alert", &alert, 1, RW);
+	jsDefineFunction(m_pWindowObj, "open", &windowOpen, 1, RW);
+	jsDefineFunction(m_pWindowObj, "setTimeout", &doNothing, 2, RW);
+	jsDefineFunction(m_pWindowObj, "setInterval", &doNothing, 2, RW);
+	jsDefineFunction(m_pWindowObj, "clearTimeout", &doNothing, 1, RW);
+	jsDefineFunction(m_pWindowObj, "clearInterval", &doNothing, 1, RW);
+	jsDefineFunction(m_pWindowObj, "focus", &doNothing, 0, RW);
+	jsDefineFunction(m_pWindowObj, "blur", &doNothing, 0, RW);
+	jsDefineFunction(m_pWindowObj, "resizeTo", &doNothing, 2, RW);
+	jsDefineFunction(m_pWindowObj, "moveTo", &doNothing, 2, RW);
+	jsDefineFunction(m_pWindowObj, "Image", &imageConstructor, 0, RW);
 	jsDefineProperty(m_pWindowObj, "onload", JSVAL_VOID,
 		0, &onloadSetter, JSPROP_ENUMERATE|JSPROP_PERMANENT
 	);
@@ -362,95 +389,93 @@ JsEnvironment::Context::Context(URI const& page_url)
 	
 	initLocationObject(location_obj, page_url);
 	
-	JSObject* nav_obj = jsDefineObject(m_pWindowObj, "navigator", &m_sNavigatorClass);
-	jsDefineFunction(nav_obj, "javaEnabled", &javaEnabled, 0);
-	jsDefineFunction(nav_obj, "cookieEnabled", &cookieEnabled, 0);
-	jsDefineProperty(nav_obj, "platform", STRING_TO_JSVAL(jsNewStringCopyZ("Win32")));
-	jsDefineProperty(nav_obj, "appCodeName", STRING_TO_JSVAL(jsNewStringCopyZ("Mozilla")));
-	jsDefineProperty(nav_obj, "appName", STRING_TO_JSVAL(jsNewStringCopyZ("Netscape")));
-	jsDefineProperty(nav_obj, "appVersion", STRING_TO_JSVAL(
-		jsNewStringCopyZ("5.0 (Windows; en-US)")
-	));
+	JSObject* nav_obj = jsDefineObject(m_pWindowObj, "navigator", &m_sNavigatorClass, RW);
+	jsDefineFunction(nav_obj, "javaEnabled", &javaEnabled, 0, RW);
+	jsDefineProperty(nav_obj, "cookieEnabled", JSVAL_FALSE, RO);
+	jsDefineProperty(nav_obj, "platform", STRING_TO_JSVAL(jsNewStringCopyZ("Win32")), RO);
+	jsDefineProperty(nav_obj, "appCodeName", STRING_TO_JSVAL(jsNewStringCopyZ("Mozilla")), RO);
+	jsDefineProperty(nav_obj, "appName", STRING_TO_JSVAL(jsNewStringCopyZ("Netscape")), RO);
+	jsDefineProperty(nav_obj, "appVersion",
+		STRING_TO_JSVAL(jsNewStringCopyZ("5.0 (Windows; en-US)")), RO
+	);
 	jsDefineProperty(nav_obj, "userAgent", STRING_TO_JSVAL(jsNewStringCopyZ(
 		"Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8) Gecko/20051111 Firefox/1.5"
-	)));
+	)), RO);
 	
 	JSObject* mimetypes_array = jsNewArrayObject(0, 0);
-	jsDefineProperty(nav_obj, "mimeTypes", OBJECT_TO_JSVAL(mimetypes_array));
+	jsDefineProperty(nav_obj, "mimeTypes", OBJECT_TO_JSVAL(mimetypes_array), RO);
 	JSObject* plugins_array = jsNewArrayObject(0, 0);
-	jsDefineProperty(nav_obj, "plugins", OBJECT_TO_JSVAL(plugins_array));
+	jsDefineProperty(nav_obj, "plugins", OBJECT_TO_JSVAL(plugins_array), RO);
 	JSObject* flash_mimetype_obj = jsDefineObject(
-		mimetypes_array, "application/x-shockwave-flash", &m_sMimetypeClass
+		mimetypes_array, "application/x-shockwave-flash", &m_sMimetypeClass, RW
 	);
 	jsval flash_mimetype_jsval = OBJECT_TO_JSVAL(flash_mimetype_obj);
 	jsSetElement(mimetypes_array, 0, &flash_mimetype_jsval);
-	jsDefineProperty(flash_mimetype_obj, "description", STRING_TO_JSVAL(
-		jsNewStringCopyZ("Shockwave Flash 7.0")
-	));
-	jsDefineProperty(flash_mimetype_obj, "type", STRING_TO_JSVAL(
-		jsNewStringCopyZ("application/x-shockwave-flash")
-	));
-	jsDefineProperty(flash_mimetype_obj, "suffixes", STRING_TO_JSVAL(
-		jsNewStringCopyZ("swf")
-	));
+	jsDefineProperty(flash_mimetype_obj, "description",
+		STRING_TO_JSVAL(jsNewStringCopyZ("Shockwave Flash 7.0")), RO
+	);
+	jsDefineProperty(flash_mimetype_obj, "type",
+		STRING_TO_JSVAL(jsNewStringCopyZ("application/x-shockwave-flash")), RO
+	);
+	jsDefineProperty(flash_mimetype_obj, "suffixes",
+		STRING_TO_JSVAL(jsNewStringCopyZ("swf")), RO
+	);
 	JSObject *flash_plugin_obj = jsDefineObject(
-		flash_mimetype_obj, "enabledPlugin", &m_sPluginClass
+		flash_mimetype_obj, "enabledPlugin", &m_sPluginClass, RO
 	);
 	jsval flash_plugin_jsval = OBJECT_TO_JSVAL(flash_plugin_obj);
 	jsSetElement(plugins_array, 0, &flash_plugin_jsval);
-	jsDefineProperty(plugins_array, "Shockwave Flash", flash_plugin_jsval);
-	jsDefineProperty(flash_plugin_obj, "name", STRING_TO_JSVAL(
-		jsNewStringCopyZ("Shockwave Flash")
-	));
-	jsDefineProperty(flash_plugin_obj, "description", STRING_TO_JSVAL(
-		jsNewStringCopyZ("Shockwave Flash 7.0")
-	));
-	jsDefineProperty(flash_plugin_obj, "length", INT_TO_JSVAL(1));
-	jsDefineFunction(flash_plugin_obj, "refresh", &doNothing, 0);
+	jsDefineProperty(plugins_array, "Shockwave Flash", flash_plugin_jsval, RW);
+	jsDefineProperty(flash_plugin_obj, "name",
+		STRING_TO_JSVAL(jsNewStringCopyZ("Shockwave Flash")), RO
+	);
+	jsDefineProperty(flash_plugin_obj, "description",
+		STRING_TO_JSVAL(jsNewStringCopyZ("Shockwave Flash 7.0")), RO
+	);
+	jsDefineProperty(flash_plugin_obj, "length", INT_TO_JSVAL(1), RO);
 	
-	JSObject* screen_obj = jsDefineObject(m_pWindowObj, "screen", &m_sScreenClass);
-	jsDefineProperty(screen_obj, "width", INT_TO_JSVAL(1024));
-	jsDefineProperty(screen_obj, "height", INT_TO_JSVAL(768));
-	jsDefineProperty(screen_obj, "availWidth", INT_TO_JSVAL(1024));
-	jsDefineProperty(screen_obj, "availHeight", INT_TO_JSVAL(738));
-	jsDefineProperty(screen_obj, "colorDepth", INT_TO_JSVAL(16));
-	jsDefineProperty(screen_obj, "pixelDepth", INT_TO_JSVAL(16));
+	JSObject* screen_obj = jsDefineObject(m_pWindowObj, "screen", &m_sScreenClass, RO);
+	jsDefineProperty(screen_obj, "width", INT_TO_JSVAL(1024), RO);
+	jsDefineProperty(screen_obj, "height", INT_TO_JSVAL(768), RO);
+	jsDefineProperty(screen_obj, "availWidth", INT_TO_JSVAL(1024), RO);
+	jsDefineProperty(screen_obj, "availHeight", INT_TO_JSVAL(738), RO);
+	jsDefineProperty(screen_obj, "colorDepth", INT_TO_JSVAL(16), RO);
+	jsDefineProperty(screen_obj, "pixelDepth", INT_TO_JSVAL(16), RO);
 	
 	JSObject *frames_obj = jsNewArrayObject(0, 0);
-	jsDefineProperty(m_pWindowObj, "frames", OBJECT_TO_JSVAL(frames_obj));
+	jsDefineProperty(m_pWindowObj, "frames", OBJECT_TO_JSVAL(frames_obj), RW);
 	
-	m_pDocumentObj = jsDefineObject(m_pWindowObj, "document", &m_sDocumentClass);
+	m_pDocumentObj = jsDefineObject(m_pWindowObj, "document", &m_sDocumentClass, RW);
 	jsDefineProperty(m_pDocumentObj, "cookie", JS_GetEmptyStringValue(m_pContext),
 		&cookieGetter, &cookieSetter, JSPROP_ENUMERATE|JSPROP_PERMANENT
 	);
-	jsDefineProperty(m_pDocumentObj, "location", OBJECT_TO_JSVAL(location_obj));
-	jsDefineProperty(m_pDocumentObj, "domain", JS_GetEmptyStringValue(m_pContext));
-	jsDefineProperty(m_pDocumentObj, "referrer", STRING_TO_JSVAL(
-		jsNewStringCopyZ("http://whatever")
-	));
+	jsDefineProperty(m_pDocumentObj, "domain", JS_GetEmptyStringValue(m_pContext), RW);
+	jsDefineProperty(m_pDocumentObj, "referrer",
+		STRING_TO_JSVAL(jsNewStringCopyZ("http://whatever")), RO
+	);
 	JSObject* images_array = jsNewArrayObject(0, 0);
-	jsDefineProperty(m_pDocumentObj, "images", OBJECT_TO_JSVAL(images_array));
-	jsDefineProperty(m_pDocumentObj, "URL", JS_GetEmptyStringValue(m_pContext));
-	jsDefineFunction(m_pDocumentObj, "write", &documentWrite, 1);
-	jsDefineFunction(m_pDocumentObj, "writeln", &documentWriteln, 1);
-	jsDefineFunction(m_pDocumentObj, "getElementById", &documentGetElementById, 1);
-	jsDefineFunction(m_pDocumentObj, "getElementsByTagName", &documentGetElementsByTagName, 1);
-	jsDefineFunction(m_pDocumentObj, "open", &doNothing, 0);
-	jsDefineFunction(m_pDocumentObj, "close", &doNothing, 0);
+	jsDefineProperty(m_pDocumentObj, "images", OBJECT_TO_JSVAL(images_array), RO);
+	jsDefineProperty(m_pDocumentObj, "URL", JS_GetEmptyStringValue(m_pContext), RO);
+	jsDefineFunction(m_pDocumentObj, "write", &documentWrite, 1, RW);
+	jsDefineFunction(m_pDocumentObj, "writeln", &documentWriteln, 1, RW);
+	jsDefineFunction(m_pDocumentObj, "getElementById", &documentGetElementById, 1, RW);
+	jsDefineFunction(m_pDocumentObj, "getElementsByTagName", &documentGetElementsByTagName, 1, RW);
+	jsDefineFunction(m_pDocumentObj, "open", &doNothing, 0, RW);
+	jsDefineFunction(m_pDocumentObj, "close", &doNothing, 0, RW);
 	jsDefineProperty(
 		m_pDocumentObj, "location", OBJECT_TO_JSVAL(location_obj),
 		0, &locationSetter, JSPROP_ENUMERATE|JSPROP_PERMANENT
 	);
 	
 	m_pFormArray = jsNewArrayObject(0, 0);
-	jsDefineProperty(m_pDocumentObj, "forms", OBJECT_TO_JSVAL(m_pFormArray));
+	jsDefineProperty(m_pDocumentObj, "forms", OBJECT_TO_JSVAL(m_pFormArray), RO);
 	
-	JSObject* body_obj = jsDefineObject(m_pDocumentObj, "body", &m_sBodyClass);
-	jsDefineProperty(body_obj, "clientWidth", INT_TO_JSVAL(1024));
-	jsDefineProperty(body_obj, "clientHeight", INT_TO_JSVAL(602));
-	jsDefineProperty(body_obj, "innerText", JS_GetEmptyStringValue(m_pContext));
-	jsDefineProperty(body_obj, "innerHTML", JS_GetEmptyStringValue(m_pContext));
-	jsDefineFunction(body_obj, "getElementsByTagName", &documentGetElementsByTagName, 1);
+	JSObject* body_obj = jsDefineObject(m_pDocumentObj, "body", &m_sBodyClass, RW);
+	jsDefineProperty(body_obj, "clientWidth", INT_TO_JSVAL(1024), RO);
+	jsDefineProperty(body_obj, "clientHeight", INT_TO_JSVAL(602), RO);
+	jsDefineProperty(body_obj, "innerText", JS_GetEmptyStringValue(m_pContext), RW);
+	jsDefineProperty(body_obj, "innerHTML", JS_GetEmptyStringValue(m_pContext), RW);
+	jsDefineFunction(body_obj, "getElementsByTagName", &documentGetElementsByTagName, 1, RW);
 }
 
 JsEnvironment::Context::~Context()
@@ -720,14 +745,6 @@ JsEnvironment::Context::javaEnabled(
 	return JS_TRUE;
 }
 
-JSBool
-JsEnvironment::Context::cookieEnabled(
-	JSContext* cx, JSObject* obj, uintN argc, jsval* argv, jsval* rval)
-{
-	*rval = JSVAL_TRUE;
-	return JS_TRUE;
-}
-
 JSBool
 JsEnvironment::Context::alert(
 	JSContext* cx, JSObject* obj, uintN argc, jsval* argv, jsval* rval)
@@ -747,19 +764,34 @@ JSBool
 JsEnvironment::Context::documentWrite(
 	JSContext* cx, JSObject* obj, uintN argc, jsval* argv, jsval* rval)
 {
-	return processDocumentWrite(cx, argc, argv, rval, false);
+	return processDocumentWrite(cx, argc, argv, rval, PRIMARY_DOCUMENT, false);
 }
 
 JSBool
 JsEnvironment::Context::documentWriteln(
 	JSContext* cx, JSObject* obj, uintN argc, jsval* argv, jsval* rval)
 {
-	return processDocumentWrite(cx, argc, argv, rval, true);
+	return processDocumentWrite(cx, argc, argv, rval, PRIMARY_DOCUMENT, true);
+}
+
+JSBool
+JsEnvironment::Context::secondaryDocumentWrite(
+	JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
+{
+	return processDocumentWrite(cx, argc, argv, rval, SECONDARY_DOCUMENT, false);
+}
+
+JSBool
+JsEnvironment::Context::secondaryDocumentWriteln(
+	JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
+{
+	return processDocumentWrite(cx, argc, argv, rval, SECONDARY_DOCUMENT, true);
 }
 
 JSBool
 JsEnvironment::Context::processDocumentWrite(
-	JSContext* cx, uintN argc, jsval* argv, jsval* rval, bool newline)
+	JSContext* cx, uintN argc, jsval* argv, jsval* rval,
+	Origin origin, bool newline)
 {
 	*rval = JSVAL_VOID;
 	if (argc > 0) {
@@ -772,7 +804,7 @@ JsEnvironment::Context::processDocumentWrite(
 			Listener* listener = getEnvContext(cx)->getListener();
 			if (listener) {
 				JsRequestSuspender suspender(cx);
-				listener->processJsOutput(data, newline);
+				listener->processJsOutput(data, origin, newline);
 			}
 		}
 	}
@@ -800,6 +832,30 @@ JsEnvironment::Context::documentGetElementById(
 		return JS_FALSE;
 	}
 	
+	JSObject* content_document = JS_DefineObject(
+		cx, el,"contentDocument", &m_sDocumentClass, 0,
+		JSPROP_ENUMERATE|JSPROP_PERMANENT
+	);
+	if (!content_document) {
+		return JS_FALSE;
+	}
+	
+	JSFunction* fun = JS_DefineFunction(
+		cx, content_document, "write", &secondaryDocumentWrite, 1,
+		JSPROP_ENUMERATE|JSPROP_PERMANENT
+	);
+	if (!fun) {
+		return JS_FALSE;
+	}
+	
+	fun = JS_DefineFunction(
+		cx, content_document, "writeln", &secondaryDocumentWriteln, 1,
+		JSPROP_ENUMERATE|JSPROP_PERMANENT
+	);
+	if (!fun) {
+		return JS_FALSE;
+	}
+	
 	ok = JS_DefineProperty(
 		cx, el, "style", JS_GetEmptyStringValue(cx),
 		0, 0, JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT
@@ -949,13 +1005,13 @@ JsEnvironment::Context::initLocationObject(JSObject* obj, URI const& url)
 	strm << url.getScheme() << ':';
 	jsDefineProperty(
 		obj, "protocol",
-		STRING_TO_JSVAL(jsNewString(strm.data().toBString()))
+		STRING_TO_JSVAL(jsNewString(strm.data().toBString())), RW
 	);
 	strm.data().clear();
 	
 	jsDefineProperty(
 		obj, "hostname",
-		STRING_TO_JSVAL(jsNewString(url.getHost()))
+		STRING_TO_JSVAL(jsNewString(url.getHost())), RW
 	);
 	
 	strm << url.getHost();
@@ -964,20 +1020,20 @@ JsEnvironment::Context::initLocationObject(JSObject* obj, URI const& url)
 	}
 	jsDefineProperty(
 		obj, "host",
-		STRING_TO_JSVAL(jsNewString(strm.data().toBString()))
+		STRING_TO_JSVAL(jsNewString(strm.data().toBString())), RW
 	);
 	strm.data().clear();
 	
 	strm << url.guessPort();
 	jsDefineProperty(
 		obj, "port",
-		STRING_TO_JSVAL(jsNewString(strm.data().toBString()))
+		STRING_TO_JSVAL(jsNewString(strm.data().toBString())), RW
 	);
 	strm.data().clear();
 	
 	jsDefineProperty(
 		obj, "pathname",
-		STRING_TO_JSVAL(jsNewString(url.getRawPath()))
+		STRING_TO_JSVAL(jsNewString(url.getRawPath())), RW
 	);
 	
 	if (!url.getRawQuery().empty()) {
@@ -985,13 +1041,13 @@ JsEnvironment::Context::initLocationObject(JSObject* obj, URI const& url)
 	}
 	jsDefineProperty(
 		obj, "hash",
-		STRING_TO_JSVAL(jsNewString(strm.data().toBString()))
+		STRING_TO_JSVAL(jsNewString(strm.data().toBString())), RW
 	);
 	strm.data().clear();
 	
 	jsDefineProperty(
 		obj, "search",
-		STRING_TO_JSVAL(jsNewString(url.getRawQuery()))
+		STRING_TO_JSVAL(jsNewString(url.getRawQuery())), RW
 	);
 	
 	jsDefineProperty(
@@ -999,7 +1055,7 @@ JsEnvironment::Context::initLocationObject(JSObject* obj, URI const& url)
 		0, &locationSetter, JSPROP_ENUMERATE|JSPROP_PERMANENT
 	);
 	
-	jsDefineFunction(obj, "toString", &locationToString, 0);
+	jsDefineFunction(obj, "toString", &locationToString, 0, RW);
 }
 
 
