@@ -6,6 +6,6 @@ if [ -z "$1" ]; then
   exit
 fi
 
-find . -type f -maxdepth 1 \
-! -regex '.*/\(\.\|Makefile\|SConscript\|update.sh\).*' -exec \
+find . -maxdepth 1 -type f \
+! -regex '.*/\(\.\|Makefile\|SConscript\|CMake\|update.sh\).*' -exec \
 sh -c "if [ -e '$1/{}' ]; then cp -fp '$1/{}' '{}'; else rm -f '{}'; fi" \;
