@@ -26,7 +26,7 @@
 
 #include "RefCountable.h"
 #include "RefCounter.h"
-#include <memory>
+#include "IntrusivePtr.h"
 
 class SplittableBuffer;
 class ErrorDescriptor;
@@ -40,7 +40,7 @@ public:
 	// postcondition: data is empty
 	virtual void appendBodyData(SplittableBuffer& data, bool eof) = 0;
 	
-	virtual void cancelRequest(std::auto_ptr<ErrorDescriptor> edesc) = 0;
+	virtual void cancelRequest(IntrusivePtr<ErrorDescriptor> const& edesc) = 0;
 };
 
 #endif
