@@ -27,24 +27,17 @@
 #include "Observer.h"
 #include "AutoClosingSAP.h"
 
+class AsyncConnectorError;
 class ACE_SOCK_Stream;
 
 class AsyncConnectorListener : public ObserverBase
 {
 public:
-	enum FailureCode {
-		FAIL_OTHER,
-		FAIL_DNS,
-		FAIL_TIMEOUT,
-		FAIL_REFUSED,
-		FAIL_UNREACHABLE
-	};
-	
 	virtual ~AsyncConnectorListener() {}
 	
 	virtual void onConnectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm) = 0;
 	
-	virtual void onConnectionFailed(FailureCode fcode, int errnum) = 0;
+	virtual void onConnectionFailed(AsyncConnectorError const& err) = 0;
 };
 
 #endif
