@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ HeuristicScore::Status
 FlashHeuristicScore::getStatus() const
 {
 	int score = getNumericScore();
-	if (score < 0 /* changed from 20 */) {
+	if (score < -20 /* changed from 20 */) {
 		return NOT_AD;
 	} else if (score < 60) {
 		return PROBABLY_NOT_AD;
@@ -86,11 +86,10 @@ FlashHeuristicScore::processUrl(URI const& abs, URI const& rel, URI const& base)
 	} else if (rship == URLS_SAME_HOST) {
 		m_score -= 20;
 	}
-	UrlStatus const status = getUrlStatus(rel);
-	if (status == URL_INNOCENT) {
-		m_score -= 10;
-	} else if (status == URL_SUSPICIOUS) {
-		m_score += 30;
+	
+	UrlFlags const flags = getUrlStatus(abs, EXPECT_FLASH);
+	if (flags & HAS_EMBEDDED_URL) {
+		m_score += 20;
 	}
 }
 
