@@ -61,7 +61,7 @@
 #include "InsensitiveEqual.h"
 #include "ArraySize.h"
 #include "GlobalState.h"
-#include "UrlPatterns.h"
+#include "CombinedUrlPatterns.h"
 #include "Debug.h"
 #include <stddef.h>
 #include <limits>
@@ -163,6 +163,10 @@ public:
 	
 	void setInnerHtmlInvolved() { m_flags |= INNER_HTML; }
 	
+	bool isFlashInvolved() const { return m_flags & FLASH; }
+	
+	void setFlashInvolved() { m_flags |= FLASH; }
+	
 	bool isRedirectInvolved() const { return m_flags & REDIRECT; }
 	
 	void setRedirectInvolved() { m_flags |= REDIRECT; }
@@ -224,7 +228,7 @@ private:
 		CSS = 8, ONLOAD_ASSIGNED = 16,
 		DISCARD_SUBPROCESSOR_OUTPUT = 32,
 		TOO_MANY_UNRELATED_SCRIPTS = 64,
-		INNER_HTML = 128
+		INNER_HTML = 128, FLASH = 256
 	};
 	enum {
 		WEBBUG_TAIL_EV1 = 1, WEBBUG_TAIL_EV2 = 2,
@@ -232,7 +236,8 @@ private:
 	};
 	static size_t const SCRIPT_OUTPUT_LIMIT = 2048;
 	
-	virtual void processJsOutput(char const* data, bool newline);
+	virtual void processJsOutput(
+		char const* data, JsEnvironment::Origin origin, bool newline);
 	
 	virtual void processOnLoadAssignment();
 	
@@ -857,6 +862,7 @@ HtmlProcessor::processOpeningEmbed(HtmlNode* embed, bool explicit_empty)
 		score.processNoscript(m_isInsideNoscript);
 		//DEBUGLOG("[score = " << score.getNumericScore() << "] EMBED: " << url);
 		if (m_pScriptContext) {
+			m_pScriptContext->setFlashInvolved();
 			score.processScriptLevel(
 				m_pScriptContext->getNestLevel(),
 				m_pScriptContext->isExternalScriptInvolved()
@@ -1135,6 +1141,8 @@ HtmlProcessor::processClosingObject(Iterator const& begin, Iterator const& end)
 	
 	BString const flash_classid("clsid:d27cdb6e-ae6d-11cf-96b8-444553540000");
 	BString const flash_mimetype("application/x-shockwave-flash");
+	BString const classid_str("classid");
+	BString const type_str("type");
 	BString const src_str("src");
 	BString const data_str("data");
 	BString const name_str("name");
@@ -1148,14 +1156,16 @@ HtmlProcessor::processClosingObject(Iterator const& begin, Iterator const& end)
 	
 	InsensitiveEqual ieq;
 	
-	if (ieq(object->getAttribute(BString("classid")), flash_classid) ||
-	    ieq(object->getAttribute(BString("type")), flash_mimetype)) {
+	if (ieq(object->getAttribute(classid_str), flash_classid) ||
+	    ieq(object->getAttribute(type_str), flash_mimetype) ||
+	    (!object->hasAttribute(classid_str) && !object->hasAttribute(type_str))) {
 		BString width_str = object->getAttribute(BString("width"));
 		BString height_str = object->getAttribute(BString("height"));
 		bool loop = false;
 		bool menu = true;
 		HtmlNode* movie_node = 0;
-		for (HtmlNode* child = object->getFirstChild(); child; child = child->getNextSibling()) {
+		for (HtmlNode* child = object->getFirstChild();
+		     child; child = child->getNextSibling()) {
 			if (child->getTagName() != param_str) {
 				continue;
 			}
@@ -1221,6 +1231,7 @@ HtmlProcessor::processFlashObject(
 	score.processOptions(loop, menu);
 	score.processNoscript(m_isInsideNoscript);
 	if (m_pScriptContext) {
+		m_pScriptContext->setFlashInvolved();
 		score.processScriptLevel(
 			m_pScriptContext->getNestLevel(),
 			m_pScriptContext->isExternalScriptInvolved()
@@ -1372,6 +1383,7 @@ HtmlProcessor::processJavaScript(HtmlNode* script, bool& nuke_adjacent_noscript)
 		} else {
 			//DEBUGLOG("Script fetching complete");
 			//DEBUGLOG("EXTERNAL_SCRIPT: " << std::endl << script_code.toString());
+			stripBOM(script_code);
 		}
 	}
 	
@@ -1617,9 +1629,11 @@ HtmlProcessor::substituteAdScript(
 {
 	auto_ptr<AbstractScriptOperation> res;
 	
-	if (context.isCssInvolved() &&
-	    !context.isInnerHtmlInvolved() &&
-	    !context.isSubprocessorOutputDiscarded()) {
+	if (m_pageCleanupLevel == Config::CLEANUP_OFF
+	    && context.isCssInvolved()
+	    && !context.isFlashInvolved()
+	    && !context.isInnerHtmlInvolved()
+	    && !context.isSubprocessorOutputDiscarded()) {
 		/*
 		Consider a case where a script outputs the following:
 		<div id="x"><a ...><img ...></a></div>
@@ -1627,6 +1641,18 @@ HtmlProcessor::substituteAdScript(
 		element x absolute positioned. In this case it's better
 		to keep the output structure, or our ad replacement will
 		appear in a wrong place, possibly covering other content.
+		
+		Q: Why !context.isFlashInvolved()?
+		A: There was a case where a big flash object was covering
+		   the content and would hide itself by calling javascript
+		   from the script it originated from (the one we are substituting).
+		   As a result, that flash object would not hide itself.
+		Q: Why !context.isInnerHtmlInvolved()?
+		A: Because el.innerHTML='<ad code>' would place an ad inside
+		   a specific element, not where the script is located.
+		   We can't achieve the same effect.
+		Q: Why !context.isSubprocessorOutputDiscarded()?
+		A: Well, because we can't use script output that was discarded.
 		*/
 		BString const prefix(
 			"<!--\r\n[BFilter] Script has been "
@@ -2392,6 +2418,20 @@ HtmlProcessor::isDynamicResource(HttpResponseMetadata const& metadata)
 	return !metadata.headers().hasHeader(BString("Last-Modified"));
 }
 
+void
+HtmlProcessor::stripBOM(SplittableBuffer& data)
+{
+	static unsigned char const utf8_bom[] = { 0xef, 0xbb, 0xbf };
+	
+	bool const has_utf8_bom = StringUtils::startsWith(
+		data.begin(), data.end(),
+		utf8_bom, utf8_bom+sizeof(utf8_bom)
+	);
+	if (has_utf8_bom) {
+		data.trimFront(sizeof(utf8_bom));
+	}
+}
+
 auto_ptr<HtmlProcessor::AbstractScriptOperation>
 HtmlProcessor::createScriptSubstitution(HtmlNode* node, AdSuspect const& ad)
 {
@@ -2437,14 +2477,17 @@ HtmlProcessor::ScriptContext::~ScriptContext()
 }
 
 void
-HtmlProcessor::ScriptContext::processJsOutput(char const* data, bool newline)
+HtmlProcessor::ScriptContext::processJsOutput(
+	char const* data, JsEnvironment::Origin origin, bool newline)
 {
 	if (m_innerHtmlLevel != 0) {
 		// Scripts inserted using innerHTML can't use document.write().
 		return;
 	}
 	
-	if (m_scriptOutputSize == 0 && strncmp(data, "-->", 3) == 0) {
+	if (origin != JsEnvironment::PRIMARY_DOCUMENT) {
+		discardSubprocessorOutput();
+	} else if (m_scriptOutputSize == 0 && strncmp(data, "-->", 3) == 0) {
 		m_webbugTailEvidence |= WEBBUG_TAIL_EV1;
 	}
 	
