@@ -38,7 +38,7 @@
 #include "IntrusivePtr.h"
 #include "State.h"
 #include "GlobalState.h"
-#include "ContentFilters.h"
+#include "AvailableFiltersOrdered.h"
 #include "URI.h"
 #include "Debug.h"
 #include <sigc++/sigc++.h>
@@ -161,10 +161,9 @@ HttpFetcher::applyRegexFilters(FilterTryList& filters, BString const& url_str)
 {
 	GlobalState::ReadAccessor state;
 	
-	typedef ContentFilters::FilterList FilterList;
-	FilterList const& avail_filters = state->contentFilters().filters();
-	FilterList::const_iterator it(avail_filters.begin());
-	FilterList::const_iterator const end(avail_filters.end());
+	AvailableFiltersOrdered const& avail_filters = state->contentFilters();
+	AvailableFiltersOrdered::iterator it(avail_filters.begin());
+	AvailableFiltersOrdered::iterator const end(avail_filters.end());
 	for (; it != end; ++it) {
 		filters.tryAppendRegexFilter(url_str, *it);
 	}
