@@ -47,6 +47,7 @@ char const* const HttpMethodRegistry::m_sMethods[] = {
 	"LOCK",
 	"MKCOL",
 	"MOVE",
+	"OPTIONS", // *
 	"POLL",
 	"POST",
 	"PROPFIND",
@@ -54,13 +55,12 @@ char const* const HttpMethodRegistry::m_sMethods[] = {
 	"PUT",
 	"SEARCH",
 	"SUBSCRIBE",
+	"TRACE", // *
 	"UNLOCK",
 	"UNSUBSCRIBE"
 };
-/* TODO:
-OPTIONS, TRACE: special support required + handle Max-Forwards header
-*/
-
+// * OPTIONS and TRACE are supported in pass-through mode only.
+//   They will work as if BFilter is not on the request path.
 
 struct HttpMethodRegistry::Comparator
 {
