@@ -36,7 +36,6 @@ ProxyDescriptor::ProxyDescriptor()
 
 ProxyDescriptor::ProxyDescriptor(ProxyDescriptor const& other)
 :	m_type(other.m_type),
-	m_configProblem(other.m_configProblem),
 	m_addr(other.m_addr),
 	m_username(other.m_username),
 	m_password(other.m_password)
@@ -54,20 +53,79 @@ ProxyDescriptor::operator=(ProxyDescriptor const& other)
 	return *this;
 }
 
+char const*
+ProxyDescriptor::getTypeAsString() const
+{
+	switch (m_type) {
+		case HTTP:
+		return "http";
+		case SOCKS4:
+		return "socks4";
+		case SOCKS4A:
+		return "socks4a";
+		case SOCKS5:
+		return "socks5";
+		case INVALID:
+		default:
+		return "";
+	}
+}
+
+ProxyDescriptor::ProxyType
+ProxyDescriptor::resolveType(std::string const& type)
+{
+	if (type == "http") {
+		return HTTP;
+	} else if (type == "socks4") {
+		return SOCKS4;
+	} else if (type == "socks4a") {
+		return SOCKS4A;
+	} else if (type == "socks5") {
+		return SOCKS5;
+	} else {
+		return INVALID;
+	}
+}
+
 void
 ProxyDescriptor::swap(ProxyDescriptor& other)
 {
 	std::swap(m_type, other.m_type);
-	m_configProblem.swap(other.m_configProblem);
 	m_addr.swap(other.m_addr);
 	m_username.swap(other.m_username);
 	m_password.swap(other.m_password);
 }
 
-void
-ProxyDescriptor::setInvalid(std::string const& config_problem)
+bool operator==(ProxyDescriptor const& lhs, ProxyDescriptor const& rhs)
 {
-	ProxyDescriptor proxy;
-	proxy.m_configProblem = config_problem;
-	proxy.swap(*this);
+	if (lhs.getType() != rhs.getType()) {
+		return false;
+	} else if (lhs.getAddr() != rhs.getAddr()) {
+		return false;
+	} else if (lhs.getUserName() != rhs.getUserName()) {
+		return false;
+	} else if (lhs.getPassword() != rhs.getPassword()) {
+		return false;
+	} else {
+		return true;
+	}
+}
+
+bool operator<(ProxyDescriptor const& lhs, ProxyDescriptor const& rhs)
+{
+	if (lhs.getType() < rhs.getType()) {
+		return true;
+	} else if (rhs.getType() < lhs.getType()) {
+		return false;
+	} else if (lhs.getAddr() < rhs.getAddr()) {
+		return true;
+	} else if (rhs.getAddr() < lhs.getAddr()) {
+		return false;
+	} else if (lhs.getUserName() < rhs.getUserName()) {
+		return true;
+	} else if (rhs.getUserName() > lhs.getUserName()) {
+		return false;
+	} else {
+		return lhs.getPassword() < rhs.getPassword();
+	}
 }
