@@ -223,14 +223,14 @@ RegexResponseFilter::registerMatch()
 	
 	string set_flag = m_ptrFilterDescriptor->setFlag();
 	if (!set_flag.empty()) {
-		getFilterChain().setFlag(
+		getFilterChain().groupLocalFlags().set(
 			m_ptrFilterDescriptor->getGroupTag(), set_flag
 		);
 	}
 	
 	string clear_flag = m_ptrFilterDescriptor->clearFlag();
 	if (!clear_flag.empty()) {
-		getFilterChain().clearFlag(
+		getFilterChain().groupLocalFlags().clear(
 			m_ptrFilterDescriptor->getGroupTag(), clear_flag
 		);
 	}
