@@ -33,6 +33,7 @@
 #include "HttpHeader.h"
 #include "Date.h"
 #include "GlobalState.h"
+#include "Conf.h"
 #include <algorithm>
 
 using namespace std;
@@ -104,18 +105,18 @@ ReplacementImage::ReplacementImage(int width, int height, Color const* border_co
 		writeImageData(strm, 0x01, width);
 		strm.swapData(temp); // now we have line data in temp
 		strm.data().append(temp);
-		writeImageDescriptor(strm, 0, height-1, width, 1);
+		writeImageDescriptor(strm, 0, height - 1, width, 1);
 		writeLocalColorTable(strm, border_color);
 		strm.data().appendDestructive(temp); // append the other line
 		
 		// write two vertical lines
-		writeImageDescriptor(strm, 0, 1, 1, height-2);
+		writeImageDescriptor(strm, 0, 1, 1, height - 2);
 		writeLocalColorTable(strm, border_color);
 		strm.swapData(temp); // store what we have so far
 		writeImageData(strm, 0x01, height-2);
 		strm.swapData(temp); // now we have line data in temp
 		strm.data().append(temp);
-		writeImageDescriptor(strm, width-1, 1, 1, height-2);
+		writeImageDescriptor(strm, width-1, 1, 1, height - 2);
 		writeLocalColorTable(strm, border_color);
 		strm.data().appendDestructive(temp); // append the other line
 	}
@@ -165,8 +166,8 @@ ReplacementImage::writeImageDescriptor(SBOutStream& out,
 {
 	char id[] = {
 		0x2c, // image separator
-		(unsigned)xpos, ((unsigned)xpos>>8),
-		(unsigned)ypos, ((unsigned)ypos>>8),
+		(unsigned)xpos, ((unsigned)xpos >> 8),
+		(unsigned)ypos, ((unsigned)ypos >> 8),
 		(unsigned)width, ((unsigned)width >> 8),
 		(unsigned)height, ((unsigned)height >> 8),
 		0x80 // flags
@@ -220,7 +221,7 @@ ReplacementImage::writeLZWCodeStream(LZWCodeStream& strm, uint8_t color_idx, siz
 	--length;
 	strm.push_back(CLEAR_CODE);
 	while (length-- > 0) {
-		if (isCodeInTable(buflen+1, tablepos)) {
+		if (isCodeInTable(buflen + 1, tablepos)) {
 			++buflen;
 		} else {
 			strm.push_back(getCodeFor(buflen, color_idx));
@@ -242,17 +243,21 @@ ReplacementImage::packLZWCodeStream(SBOutStream& out, LZWCodeStream const& strm)
 	char block[255];
 	size_t block_size = 0;
 	int code_bits = 3;
-	unsigned int max_code = (1<<3)-1;
+	unsigned int max_code = (1 << 3) - 1;
 	unsigned char cur_byte = 0;
 	int cur_byte_bits_occupied = 0;
 	out << uint8_t(0x02); // LZW code size
-	LZWCodeStream::const_iterator iter = strm.begin();
-	for (; iter != strm.end(); ++iter) {
-		LZWCode code = *iter;
+	LZWCodeStream::const_iterator it(strm.begin());
+	LZWCodeStream::const_iterator const end(strm.end());
+	for (; it != end; ++it) {
+		LZWCode code = *it;
 		int bits_injected = 0;
 		while (bits_injected < code_bits) {
 			cur_byte |= ((code >> bits_injected) << cur_byte_bits_occupied);
-			int injection = std::min(code_bits-bits_injected, 8-cur_byte_bits_occupied);
+			int injection = std::min(
+				code_bits - bits_injected,
+				8 - cur_byte_bits_occupied
+			);
 			bits_injected += injection;
 			cur_byte_bits_occupied += injection;
 			if (cur_byte_bits_occupied == 8) {
@@ -268,9 +273,9 @@ ReplacementImage::packLZWCodeStream(SBOutStream& out, LZWCodeStream const& strm)
 		}
 		if (code == CLEAR_CODE) {
 			code_bits = 3;
-			max_code = (1<<3)-1;
+			max_code = (1 << 3) - 1;
 		} else if (code >= max_code) {
-			max_code = (1 << ++code_bits)-1;
+			max_code = (1 << ++code_bits) - 1;
 		}
 	}
 	if (cur_byte_bits_occupied > 0) {
@@ -289,7 +294,7 @@ ReplacementImage::isCodeInTable(size_t seqlen, size_t tablepos)
 	if (seqlen == 1) {
 		return true;
 	} else {
-		return (tablepos > seqlen-2);
+		return (tablepos > seqlen - 2);
 	}
 }
 
