@@ -28,7 +28,7 @@
 #include "SplittableBuffer.h"
 #include "BString.h"
 #include <stddef.h>
-#include <deque>
+#include <vector>
 #include <memory>
 
 class Color;
@@ -50,7 +50,7 @@ public:
 		bool is_head_response, int width, int height, int status_code);
 private:
 	typedef uint16_t LZWCode;
-	typedef std::deque<LZWCode> LZWCodeStream;
+	typedef std::vector<LZWCode> LZWCodeStream;
 	
 	static void writeScreenDescriptor(SBOutStream& out, int width, int height);
 	
