@@ -146,21 +146,13 @@ bool
 ResponseFilterChain::isFlagSet(
 	FilterGroupTag const& group, std::string const& flag) const
 {
-	return (m_flags.find(FilterFlag(group, flag)) != m_flags.end());
-}
-
-void
-ResponseFilterChain::setFlag(
-	FilterGroupTag const& group, std::string const& flag)
-{
-	m_flags.insert(FilterFlag(group, flag));
-}
-
-void
-ResponseFilterChain::clearFlag(
-	FilterGroupTag const& group, std::string const& flag)
-{
-	m_flags.erase(FilterFlag(group, flag));
+	if (m_globalFlags.isSet(flag)) {
+		return true;
+	} else if (m_groupLocalFlags.isSet(group, flag)) {
+		return true;
+	} else {
+		return false;
+	}
 }
 
 
