@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,13 +29,14 @@
 #include "NonCopyable.h"
 #include "IntrusivePtr.h"
 #include "FilterGroupTag.h"
+#include "FilterGlobalFlags.h"
+#include "FilterGroupFlags.h"
 #include "RequestTag.h"
 #include <memory>
 #include <string>
 #include <deque>
 #include <set>
 #include <map>
-#include <utility>
 
 class AbstractResponseFilter;
 class HttpResponseMetadata;
@@ -86,12 +87,16 @@ public:
 	
 	IntrusivePtr<AbstractResponseFilter> nextFilter(HttpResponseMetadata const& metadata);
 	
+	FilterGlobalFlags& globalFlags() { return m_globalFlags; }
+	
+	FilterGlobalFlags const& globalFlags() const { return m_globalFlags; }
+	
+	FilterGroupFlags& groupLocalFlags() { return m_groupLocalFlags; }
+	
+	FilterGroupFlags const& groupLocalFlags() const { return m_groupLocalFlags; }
+	
 	bool isFlagSet(FilterGroupTag const& group, std::string const& flag) const;
 	
-	void setFlag(FilterGroupTag const& group, std::string const& flag);
-	
-	void clearFlag(FilterGroupTag const& group, std::string const& flag);
-	
 	DownloadProgress const& getDownloadProgress() const { return m_requestTag->downloadProgress(); }
 	
 	JsFilterContext& getJsContextFor(FilterGroupTag const& group);
@@ -100,7 +105,6 @@ private:
 	class RcJsFilterContext;
 
 	typedef void (*Constructor)(ResponseFilterChain&, HttpResponseMetadata const&);
-	typedef std::pair<FilterGroupTag, std::string> FilterFlag;
 	
 	ServiceContext& m_rContext;
 	ConstRequestPtr m_ptrRequest;
@@ -111,8 +115,9 @@ private:
 	IntrusivePtr<AbstractResponseFilter> m_ptrTerminator;
 	IntrusivePtr<AbstractResponseFilter> m_ptrFirstFilter;
 	std::deque<IntrusivePtr<AbstractResponseFilter> > m_filterQueue;
-	std::set<FilterFlag> m_flags;
-	std::map<FilterGroupTag, IntrusivePtr<RcJsFilterContext> > m_jsContexts;
+	FilterGlobalFlags m_globalFlags;
+	FilterGroupFlags m_groupLocalFlags;
+	std::map<FilterGroupTag, IntrusivePtr<RcJsFilterContext>, FilterGroupTag::EquivComp> m_jsContexts;
 };
 
 #endif
