@@ -49,9 +49,12 @@
 #include "HttpStreamWriter.h"
 #include "HttpMessageShaper.h"
 #include "GlobalState.h"
+#include "Conf.h"
+#include "Forwarding.h"
 #include "ServiceContext.h"
 #include "SymbolicInetAddr.h"
 #include "ProxyDescriptor.h"
+#include "ConnectionRoute.h"
 #include "DownloadProgress.h"
 #include "URI.h"
 #include "TimeDelta.h"
@@ -60,10 +63,12 @@
 #include "NetworkActivityReporter.h"
 #include "RequestLog.h"
 #include "RequestTag.h"
+#include "AsyncConnectorError.h"
 #include "SocksError.h"
 #include "Debug.h"
 #include <algorithm>
 #include <list>
+#include <vector>
 #include <cassert>
 
 using namespace std;
@@ -103,35 +108,49 @@ public:
 	
 	bool restart();
 	
-	void setErrorResponse(IntrusivePtr<ErrorDescriptor> const& edesc) { m_ptrErrDesc = edesc; }
+	void setErrorResponse(IntrusivePtr<ErrorDescriptor> const& edesc) {
+		m_ptrErrDesc = edesc;
+	}
 	
-	IntrusivePtr<ErrorDescriptor> const& errorDescriptor() { return m_ptrErrDesc; }
+	IntrusivePtr<ErrorDescriptor> const& errorDescriptor() {
+		return m_ptrErrDesc;
+	}
 	
 	std::string const& getErrorMsg() const { return m_errorMsg; }
 	
-	int getClientRequestId() const { return m_ptrOrigMetadata->getClientRequestId(); }
+	int getClientRequestId() const {
+		return m_ptrOrigMetadata->getClientRequestId();
+	}
 	
-	ConstRequestPtr const& getOrigMetadata() const { return m_ptrOrigMetadata; }
+	ConstRequestPtr const& getOrigMetadata() const {
+		return m_ptrOrigMetadata;
+	}
 	
-	HttpRequestLine const& getRequestLine() const { return m_ptrOrigMetadata->requestLine(); }
+	HttpRequestLine const& getRequestLine() const {
+		return m_ptrOrigMetadata->requestLine();
+	}
 	
-	URI const& getRequestURI() const { return m_ptrOrigMetadata->requestLine().getURI(); }
+	URI const& getRequestURI() const {
+		return getRequestLine().getURI();
+	}
 	
-	HttpHeadersCollection const& getRequestHeaders() const { return m_ptrOrigMetadata->headers(); }
+	HttpHeadersCollection const& getRequestHeaders() const {
+		return m_ptrOrigMetadata->headers();
+	}
 	
-	HttpRequestMetadata const& getOutgoingMetadata() const { return *m_ptrOutgoingMetadata; }
+	HttpRequestMetadata const& getOutgoingMetadata() const {
+		return *m_ptrOutgoingMetadata;
+	}
 	
-	bool isConnectRequest() const { return m_ptrOrigMetadata->requestLine().getMethod() == BString("CONNECT"); }
+	bool isConnectRequest() const {
+		return getRequestLine().getMethod() == BString("CONNECT");
+	}
 	
 	RequestTag const& getTag() const { return m_tag; }
 	
-	bool isGoingThroughHttpProxy() const { return m_nextHopProxy.getType() == ProxyDescriptor::HTTP; }
+	ConnectionRoute const& getConnectionRoute() const { return m_connRoute; }
 	
-	ProxyDescriptor const* getNextHopProxy() const;
-	
-	SymbolicInetAddr const& getNextHopAddr() const;
-	
-	SymbolicInetAddr const& getHttpAddr() const { return m_httpAddr; }
+	bool isGoingThroughHttpProxy() const { return m_connDestIsHttpProxy; }
 	
 	void updateRoute();
 protected:
@@ -147,7 +166,7 @@ private:
 	
 	void maybeReportClientIP(Config const& conf);
 	
-	void updateRoute(Config const& conf);
+	void updateRoute(Forwarding::Resolver const& resolver);
 	
 	ServiceContext& m_rContext;
 	ServerWeakPtr m_ptrServer;
@@ -155,8 +174,8 @@ private:
 	std::auto_ptr<HttpRequestMetadata> m_ptrOutgoingMetadata;
 	RequestTag m_tag;
 	std::auto_ptr<ResponseFilterChain> m_ptrFilterChain;
-	ProxyDescriptor m_nextHopProxy;
-	SymbolicInetAddr m_httpAddr; // http proxy or server address
+	ConnectionRoute m_connRoute;
+	bool m_connDestIsHttpProxy;
 	IntrusivePtr<ErrorDescriptor> m_ptrErrDesc;
 	std::string m_errorMsg;
 	SplittableBuffer m_data;
@@ -179,15 +198,29 @@ public:
 	
 	~PhantomRequest();
 	
-	AbstractResponseHandler& responseHandler() { return *m_ptrResponseHandler; }
+	AbstractResponseHandler& responseHandler() {
+		return *m_ptrResponseHandler;
+	}
 	
-	CraftedResponseGenerator& responseGenerator() { return m_responseGenerator; }
+	CraftedResponseGenerator& responseGenerator() {
+		return m_responseGenerator;
+	}
 	
-	URI const& getRequestURI() const { return m_ptrOrigMetadata->requestLine().getURI(); }
+	HttpRequestLine const& getRequestLine() const {
+		return m_ptrOrigMetadata->requestLine();
+	}
 	
-	int getClientRequestId() const { return m_ptrOrigMetadata->getClientRequestId(); }
+	URI const& getRequestURI() const {
+		return getRequestLine().getURI();
+	}
 	
-	bool isHeadRequest() const { return m_ptrOrigMetadata->requestLine().getMethod() == BString("HEAD"); }
+	int getClientRequestId() const {
+		return m_ptrOrigMetadata->getClientRequestId();
+	}
+	
+	bool isHeadRequest() const {
+		return getRequestLine().getMethod() == BString("HEAD");
+	}
 	
 	RequestTag const& getTag() const { return m_tag; }
 private:
@@ -336,7 +369,7 @@ void
 Server::reset()
 {
 	storeConnection();
-	abortInProgressConnection();
+	m_connector.abort();
 	assert(!m_ptrConnection.get());
 	assert(!m_handlerId);
 	assert(!m_readTimer.isActive());
@@ -430,8 +463,8 @@ Server::updatePipeline()
 				//DEBUGLOG("The previous request is not complete");
 				break;
 			}
-			if (!m_ptrConnection->isSameAddress(next_req.getHttpAddr())) {
-				//DEBUGLOG("Request to a different address");
+			if (m_ptrConnection->getRoute() != next_req.getConnectionRoute()) {
+				//DEBUGLOG("Request to a different address or through a different proxy chain");
 				break;
 			}
 			if (!m_ptrConnection->isExpectedToPersist()) {
@@ -513,7 +546,7 @@ Server::validateRequestExpectations(Request& req)
 		list<HttpHeaderElement>::iterator it = structure.elements().begin();
 		list<HttpHeaderElement>::iterator const end = structure.elements().end();
 		for (; it != end; ++it) {
-			if (InsensitiveEqual()(it->getName(), BString("100-continue"))) {
+			if (!InsensitiveEqual()(it->getName(), BString("100-continue"))) {
 				req.setErrorResponse(IntrusivePtr<ErrorDescriptor>(
 					new ErrorDescriptor(
 						ErrorCodes::EXPECTATION_FAILED,
@@ -567,7 +600,7 @@ Server::isNewConnectionRequiredFor(Request const& req) const
 	if (!m_ptrConnection->isExpectedToPersist()) {
 		return true;
 	}
-	if (!m_ptrConnection->isSameAddress(req.getHttpAddr())) {
+	if (m_ptrConnection->getRoute() != req.getConnectionRoute()) {
 		return true;
 	}
 	if (!req.isRestartable()) {
@@ -585,7 +618,7 @@ Server::requestConnectionFor(Request const& req)
 		// (requests with bodies)
 		auto_ptr<ServerConnection> conn(
 			ServerConnectionPool::instance()->retrieveConnection(
-				req.getHttpAddr()
+				req.getConnectionRoute()
 			)
 		);
 		if (conn.get()) {
@@ -595,11 +628,10 @@ Server::requestConnectionFor(Request const& req)
 		}
 	}
 	
-	//DEBUGLOG("Initiating a new server (or proxy) connection");
-	abortInProgressConnection(); // just in case
+	//DEBUGLOG("Initiating a new connection");
 	m_connector.initiate(
-		*this, m_rContext.reactor(), req.getNextHopAddr(),
-		&m_timeouts.getConnectTimeout()
+		*this, m_rContext.reactor(), req.getConnectionRoute(),
+		&m_timeouts.getConnectTimeout() // FIXME: need a different timeout
 	);
 }
 
@@ -976,19 +1008,6 @@ Server::retrieveConnection()
 	return conn;
 }
 
-bool
-Server::isConnectionInProgress() const
-{
-	return m_connector.isInProgress() || m_socksConnector.isInProgress();
-}
-
-void
-Server::abortInProgressConnection()
-{
-	m_connector.abort();
-	m_socksConnector.abort();
-}
-
 void
 Server::restartPendingRequests()
 {
@@ -997,7 +1016,7 @@ Server::restartPendingRequests()
 			m_requestQueue[i]->restart();
 		}
 		m_pipelineSize = 0;
-		abortInProgressConnection();
+		m_connector.abort();
 	}
 }
 
@@ -1080,7 +1099,7 @@ Server::updateState()
 		if (isArtificialResponseRequired()) {
 			m_delayedActionTimer.ensureActive();
 		} else if (m_pipelineSize > 0 && !m_ptrConnection.get() &&
-		           !isConnectionInProgress()) {
+		           !m_connector.isInProgress()) {
 			assert(m_pipelineSize == 1);
 			Request& req = *m_requestQueue.front();
 			if (req.hasData() || req.isConnectRequest()) {
@@ -1291,46 +1310,15 @@ Server::eventGotMetadata(auto_ptr<HttpResponseMetadata> metadata, bool is_persis
 }
 
 void
-Server::onConnectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& strm)
+Server::onConnectionEstablished(AutoClosingSAP<ACE_SOCK_Stream>& conn)
 {
 	//DEBUGLOG("Connection established");
-	assert(m_pipelineSize > 0);
-	assert(m_requestQueue.size() >= m_pipelineSize);
-	
-	Request& req = *m_requestQueue.front();
-	ProxyDescriptor const* proxy = req.getNextHopProxy();
-	
-	if (!proxy || proxy->getType() == ProxyDescriptor::HTTP) {
-		processNewHttpConnection(strm);
-		return;
-	}
-	
-	SocksAsyncConnector::SocksType socks_type;
-	switch (proxy->getType()) {
-		case ProxyDescriptor::SOCKS4:
-		socks_type = SocksAsyncConnector::SOCKS4;
-		break;
-		case ProxyDescriptor::SOCKS4A:
-		socks_type = SocksAsyncConnector::SOCKS4A;
-		break;
-		case ProxyDescriptor::SOCKS5:
-		socks_type = SocksAsyncConnector::SOCKS5;
-		break;
-		default:
-		assert(0 && "unknown proxy type");
-		return;
-	}
-	
-	m_socksConnector.initiate(
-		socks_type, *this, m_rContext.reactor(),
-		strm, req.getHttpAddr(),
-		proxy->getUserName(), proxy->getPassword(),
-		&m_timeouts.getSocksConnectTimeout()
-	);
+	processNewHttpConnection(conn);
 }
 
 void
-Server::onConnectionFailed(FailureCode fcode, int errnum)
+Server::onConnectionFailed(
+	AsyncConnectorError const& err, SymbolicInetAddr const& addr)
 {
 	DEBUGLOG("Connection failed");
 	assert(m_pipelineSize > 0);
@@ -1340,67 +1328,56 @@ Server::onConnectionFailed(FailureCode fcode, int errnum)
 	Request const& req = *m_requestQueue.front();
 	URI const& uri = req.getRequestURI();
 	
-	switch (fcode) {
-		case FAIL_DNS: {
+	switch (err.getCode()) {
+		case AsyncConnectorError::NAME_RESOLUTION_FAILED: {
 			answerWithError(IntrusivePtr<ErrorDescriptor>(
 				new ErrorDescriptor(
-					ErrorCodes::DNS_ERROR,
-					"could not resolve host name",
+					ErrorCodes::DNS_ERROR, err.toString(),
 					sigc::bind(
 						sigc::ptr_fun(&ErrorResponse::errHostNameResolution),
-						uri.toString(), req.getNextHopAddr().getHost()
+						uri.toString(), addr.getHost()
 					)
 				)
 			));
 			break;
 		}
-		case FAIL_TIMEOUT: {
+		case AsyncConnectorError::CONNECTION_TIMEOUT: {
 			answerWithError(IntrusivePtr<ErrorDescriptor>(
 				new ErrorDescriptor(
-					ErrorCodes::CONNECT_TIMEOUT,
-					"connection timeout",
+					ErrorCodes::CONNECT_TIMEOUT, err.toString(),
 					sigc::bind(
 						sigc::ptr_fun(&ErrorResponse::errConnectionTimeout),
-						uri.toString(),
-						req.getNextHopAddr().getHost(),
-						req.getNextHopAddr().getPort()
+						uri.toString(), addr.getHost(), addr.getPort()
 					)
 				)
 			));
 			break;
 		}
-		case FAIL_REFUSED:
-		case FAIL_UNREACHABLE: {
-			char const* syserr = (fcode == FAIL_REFUSED)
-				? "connection refused"
-				: "destination host unreachable";
+		case AsyncConnectorError::CONNECTION_REFUSED:
+		case AsyncConnectorError::DESTINATION_UNREACHABLE: {
 			answerWithError(IntrusivePtr<ErrorDescriptor>(
 				new ErrorDescriptor(
 					ErrorCodes::CONNECT_FAILED,
 					"connection failed",
 					sigc::bind(
 						sigc::ptr_fun(&ErrorResponse::errConnectionFailed),
-						uri.toString(), req.getNextHopAddr().getHost(),
-						req.getNextHopAddr().getPort(), syserr
+						uri.toString(), addr.getHost(),
+						addr.getPort(), err.toString()
 					)
 				)
 			));
 			break;
 		}
-		case FAIL_OTHER:
+		case AsyncConnectorError::GENERIC_ERROR:
 		default: {
-			char const* syserr = ACE::sock_error(errnum);
-			if (!syserr) {
-				syserr = ACE_OS::strerror(errnum);
-			}
 			answerWithError(IntrusivePtr<ErrorDescriptor>(
 				new ErrorDescriptor(
 					ErrorCodes::CONNECT_FAILED,
 					"connection failed",
 					sigc::bind(
 						sigc::ptr_fun(&ErrorResponse::errConnectionFailed),
-						uri.toString(), req.getNextHopAddr().getHost(),
-						req.getNextHopAddr().getPort(), syserr
+						uri.toString(), addr.getHost(),
+						addr.getPort(), err.toString()
 					)
 				)
 			));
@@ -1411,32 +1388,38 @@ Server::onConnectionFailed(FailureCode fcode, int errnum)
 }
 
 void
-Server::onConnThroughSocksDone(AutoClosingSAP<ACE_SOCK_Stream>& strm)
+Server::onConnectionFailed(SocksError const& err, ProxyDescriptor const& proxy)
 {
-	//DEBUGLOG("Connection through socks proxy established");
-	processNewHttpConnection(strm);
+	DEBUGLOG("Forwarding through a socks proxy failed");
+	handleForwardingProblem(err.toString(), proxy);
 }
 
 void
-Server::onConnThroughSocksFailed(SocksError const& err)
+Server::onConnectionFailed(std::string const& err_msg, ProxyDescriptor const& proxy)
+{
+	DEBUGLOG("Forwarding through a proxy failed");
+	handleForwardingProblem(err_msg, proxy);
+}
+
+void
+Server::handleForwardingProblem(
+	std::string const& problem, ProxyDescriptor const& proxy)
 {
-	DEBUGLOG("Connection through socks proxy failed");
 	assert(m_pipelineSize > 0);
 	assert(m_requestQueue.size() >= m_pipelineSize);
 	
 	m_ptrConnection.reset(0); // Do we need this?
 	Request const& req = *m_requestQueue.front();
 	URI const& uri = req.getRequestURI();
-	std::string problem(err.toString());
 	
 	answerWithError(IntrusivePtr<ErrorDescriptor>(
 		new ErrorDescriptor(
-			ErrorCodes::SOCKS_PROXY_ERROR,
+			ErrorCodes::ERROR_FORWARDING_THROUGH_PROXY,
 			problem,
 			sigc::bind(
-				sigc::ptr_fun(&ErrorResponse::errSocksProxyError),
-				uri.toString(), req.getNextHopAddr().getHost(),
-				req.getNextHopAddr().getPort(), problem
+				sigc::ptr_fun(&ErrorResponse::errForwardingThroughProxy),
+				uri.toString(), problem, proxy.getTypeAsString(),
+				proxy.getAddr().getHost(), proxy.getAddr().getPort()
 			)
 		)
 	));
@@ -1502,7 +1485,7 @@ Server::processNewHttpConnection(AutoClosingSAP<ACE_SOCK_Stream>& strm)
 	
 	Request const& req = *m_requestQueue.front();
 	auto_ptr<ServerConnection> conn(
-		new ServerConnection(req.getHttpAddr(), strm)
+		new ServerConnection(req.getConnectionRoute(), strm)
 	);
 	processNewHttpConnection(conn);
 }
@@ -1583,7 +1566,7 @@ Server::onDelayedAction()
 		return;
 	}
 	
-	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !isConnectionInProgress()) {
+	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !m_connector.isInProgress()) {
 		assert(m_pipelineSize == 1);
 		m_requestQueue.front()->updateRoute();
 		// This may trigger a proxy configuration error, which
@@ -1592,11 +1575,12 @@ Server::onDelayedAction()
 	
 	if (isArtificialResponseRequired()) {
 		processArtificialResponses();
+		// This may result in a recursive main loop invocation.
 	}
 	
 	assert(!isArtificialResponseRequired());
 	
-	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !isConnectionInProgress()) {
+	if (m_pipelineSize > 0 && !m_ptrConnection.get() && !m_connector.isInProgress()) {
 		assert(m_pipelineSize == 1);
 		Request& req = *m_requestQueue.front();
 		if (req.hasData() || req.isConnectRequest()) {
@@ -1623,7 +1607,8 @@ Server::Request::Request(
 		context, m_ptrOrigMetadata, request_tag,
 		filter_try_list, response_handler, *this
 	)),
-	m_httpAddr(string(), -1),
+	m_connRoute(SymbolicInetAddr(std::string(), -1)),
+	m_connDestIsHttpProxy(false),
 	m_isEOF(false),
 	m_isConsumingStarted(false),
 	m_isConstructionDone(false),
@@ -1632,7 +1617,7 @@ Server::Request::Request(
 	{
 		GlobalState::ReadAccessor global_state;
 		maybeReportClientIP(global_state->config());
-		updateRoute(global_state->config());
+		updateRoute(global_state->forwardingResolver());
 	}
 	
 	HttpStreamWriter::HttpProxy const http_proxy = (
@@ -1652,66 +1637,49 @@ Server::Request::~Request()
 {
 }
 
-ProxyDescriptor const*
-Server::Request::getNextHopProxy() const
-{
-	if (m_nextHopProxy.getType() == ProxyDescriptor::INVALID) {
-		return 0;
-	} else {
-		return &m_nextHopProxy;
-	}
-}
-
-SymbolicInetAddr const&
-Server::Request::getNextHopAddr() const
-{
-	switch (m_nextHopProxy.getType()) {
-		case ProxyDescriptor::INVALID:
-		case ProxyDescriptor::HTTP:
-		return m_httpAddr;
-		default:
-		return m_nextHopProxy.getAddr();
-	}
-}
-
 void
 Server::Request::updateRoute()
 {
-	updateRoute(GlobalState::ReadAccessor()->config());
+	updateRoute(GlobalState::ReadAccessor()->forwardingResolver());
 }
 
 void
-Server::Request::updateRoute(Config const& conf)
+Server::Request::updateRoute(Forwarding::Resolver const& resolver)
 {
+	Forwarding::ProxyChainConstPtr proxy_chain;
 	URI const& uri = getRequestURI();
-	std::string host = uri.getHost().toStdString();
-	auto_ptr<ProxyDescriptor> proxy(conf.getProxyFor(host));
 	
-	if (proxy.get() && proxy->getType() == ProxyDescriptor::HTTP) {
-		m_httpAddr = proxy->getAddr();
-	} else {
-		m_httpAddr.set(host, uri.guessPort());
-	}
-	
-	if (proxy.get()) {
-		m_nextHopProxy = *proxy;
-	} else {
-		ProxyDescriptor().swap(m_nextHopProxy);
-		return;
-	}
-	
-	if (proxy->getType() == ProxyDescriptor::INVALID) {
+	try {
+		proxy_chain = resolver.resolve(uri);
+	} catch (Forwarding::ResolverException const& e) {
+		std::string const problem(e.what());
 		setErrorResponse(IntrusivePtr<ErrorDescriptor>(
 			new ErrorDescriptor(
 				ErrorCodes::REMOTE_PROXY_CONFIG_PROBLEM,
-				proxy->getProblem(),
+				problem,
 				sigc::bind(
 					sigc::ptr_fun(&ErrorResponse::errRemoteProxyConfig),
-					getRequestURI().toString(),
-					proxy->getProblem()
+					uri.toString(), problem
 				)
 			)
 		));
+		return;
+	}
+	
+	if (!proxy_chain->empty()
+	    && proxy_chain->back().getType() == ProxyDescriptor::HTTP) {
+		ConnectionRoute(
+			proxy_chain->begin(), --proxy_chain->end(),
+			proxy_chain->back().getAddr()
+		).swap(m_connRoute);
+		m_connDestIsHttpProxy = true;
+	} else {
+		SymbolicInetAddr const dest(
+			uri.getHost().toStdString(),
+			uri.guessPort()
+		);
+		ConnectionRoute(*proxy_chain, dest).swap(m_connRoute);
+		m_connDestIsHttpProxy = false;
 	}
 }
 
