@@ -79,6 +79,28 @@ RegexFilterDescriptor::RegexFilterDescriptor(RegexFilterDescriptor const& other)
 	}
 }
 
+RegexFilterDescriptor::RegexFilterDescriptor(
+	FilterTag const& tag, FilterGroupTag const& group_tag,
+	RegexFilterDescriptor const& prototype)
+:	m_tag(tag),
+	m_groupTag(group_tag),
+	m_name(prototype.m_name),
+	m_order(prototype.m_order),
+	m_matchCountLimit(prototype.m_matchCountLimit),
+	m_ptrUrlPattern(prototype.m_ptrUrlPattern),
+	m_ptrContentTypePattern(prototype.m_ptrContentTypePattern),
+	m_ptrSearchPattern(prototype.m_ptrSearchPattern),
+	m_replacementType(prototype.m_replacementType),
+	m_ifFlag(prototype.m_ifFlag),
+	m_setFlag(prototype.m_setFlag),
+	m_clearFlag(prototype.m_clearFlag),
+	m_isEnabled(prototype.m_isEnabled)
+{
+	if (prototype.m_ptrReplacement.get()) {
+		m_ptrReplacement.reset(new string(*prototype.m_ptrReplacement));
+	}
+}
+
 RegexFilterDescriptor::~RegexFilterDescriptor()
 {
 }
@@ -153,3 +175,15 @@ RegexFilterDescriptor::isValid() const
 	}
 	return true;
 }
+
+bool operator<(RegexFilterDescriptor const& lhs, RegexFilterDescriptor const& rhs)
+{
+	if (lhs.order() < rhs.order()) {
+		return true;
+	} else if (rhs.order() < lhs.order()) {
+		return false;
+	} else {
+		FilterGroupTag::OrderComp comp;
+		return comp(lhs.getGroupTag(), rhs.getGroupTag());
+	}
+}
