@@ -26,13 +26,16 @@
 #include "Conf.h"
 #include "ConfError.h"
 #include "ConfErrorHandler.h"
-#include "ConfigFile.h"
+#include "ConfigFileStructure.h"
 #include "UrlPatterns.h"
-#include "UrlPatternsFile.h"
-#include "ContentFilters.h"
+#include "UrlsFileStructure.h"
+#include "ContentFilterList.h"
+#include "AvailableContentFilters.h"
+#include "AvailableFiltersOrdered.h"
+#include "Forwarding.h"
 #include "FilterGroupTag.h"
-#include "ContentFiltersFile.h"
-#include "CombinedUrlPatterns.h"
+#include "FilterFileStructure.h"
+#include "FilterJsLogger.h"
 #include "RegexFilterDescriptor.h"
 #include "GlobalState.h"
 #include "ConnAcceptor.h"
@@ -43,6 +46,7 @@
 #include "AutoClosingSAP.h"
 #include "RefCountableSAP.h"
 #include "StringUtils.h"
+#include "Color.h"
 #include "ArraySize.h"
 #include "Alarm.h"
 #include "types.h"
@@ -68,7 +72,6 @@
 #include <fstream>
 #include <sstream>
 #include <string>
-#include <deque>
 #include <vector>
 #include <list>
 #include <cstdlib>
@@ -120,43 +123,86 @@ private:
 };
 
 
-class Daemon::ConfigLoader
+class FileOps
 {
 public:
-	bool load(std::string const& confdir,
-		Config& target, ConfigFile& target_structure);
+	static bool readFile(std::string const& file_path,
+		std::string& target, bool log_errors = true);
+};
+
+
+class Daemon::ConfigLoader
+{
+	DECLARE_NON_COPYABLE(ConfigLoader)
+public:
+	ConfigLoader(std::string const& file_path);
+	
+	~ConfigLoader();
+	
+	/**
+	 * \brief Load and apply, but don't bind to listen addresses.
+	 */
+	bool loadAndApply();
+	
+	bool load(Config& new_config, ConfigFileStructure& new_structure);
+private:
+	std::string m_filePath;
+};
+
+
+class Daemon::ForwardingConfigLoader
+{
+	DECLARE_NON_COPYABLE(ForwardingConfigLoader)
+public:
+	ForwardingConfigLoader(std::string const& file_path);
+	
+	~ForwardingConfigLoader();
+	
+	bool loadAndApply(ObsoleteForwardingInfo const& fallback);
+private:
+	void applyFallback(ObsoleteForwardingInfo const& fallback);
+	
+	std::string m_filePath;
 };
 
 
 class Daemon::UrlPatternsLoader
 {
+	DECLARE_NON_COPYABLE(UrlPatternsLoader)
 public:
-	bool load(std::string const& confdir);
+	enum Which { URLS, URLS_LOCAL };
 	
-	CombinedUrlPatterns const& getCombinedPatterns() const {
-		return m_combinedPatterns;
-	}
+	UrlPatternsLoader(Which which, std::string const& file_path);
+	
+	~UrlPatternsLoader();
+	
+	bool loadAndApply();
 private:
-	bool load(std::string const& confdir,
-		std::string const& fname, UrlPatterns& target);
-	
-	CombinedUrlPatterns m_combinedPatterns;
+	std::string m_filePath;
+	Which m_which;
 };
 
 
 class Daemon::ContentFiltersLoader
 {
+	DECLARE_NON_COPYABLE(ContentFiltersLoader)
 public:
-	bool load(std::string const& confdir);
+	ContentFiltersLoader(std::string const& filters_dir);
 	
-	ContentFilters const& getFilters() const { return m_filters; }
+	~ContentFiltersLoader();
+	
+	void loadAndApply();
 private:
+	void loadFilter(std::string const& fname);
+	
+	std::vector<std::string> readFilterFileNames();
+	
 	void loadEnabledFilterList(
-		ContentFilters& filters, std::string const& abs_fname);
+		ContentFilterList& filters,
+		std::string const& enabled_file_path);
 	
-	void appendEnabledFilters(ContentFilters const& filters);
-	
-	ContentFilters m_filters;
+	std::string m_filtersDir;
+	AvailableContentFilters m_filters;
 };
 
 
@@ -191,7 +237,7 @@ private:
 	SymbolicInetAddr const m_defaultListenAddr;
 	SymbolicInetAddr const m_badAddr;
 	Config m_config;
-	ConfigFile m_configFile;
+	ConfigFileStructure m_configFile;
 };
 
 #endif // ENABLE_RECONFIGURE
@@ -249,75 +295,171 @@ Daemon::ConfigErrorHandler::handleError(ConfError const& err)
 }
 
 
-/* ====================== Daemon::ConfigLoader =========================*/
+/* ========================= Daemon::FileOps ===========================*/
 
 bool
-Daemon::ConfigLoader::load(
-	std::string const& confdir, Config& target, ConfigFile& target_structure)
+FileOps::readFile(std::string const& file_path,
+	std::string& target, bool log_errors)
 {
-	string const fname("config");
-	string const abs_fname(confdir+"/"+fname);
-	ifstream istrm(abs_fname.c_str());
-	if (istrm.fail()) {
-		cerr << "Could not open " << abs_fname << endl;
-		return false;
-	}
-	ostringstream ostrm;
-	ostrm << istrm.rdbuf();
-	if (istrm.bad()) {
-		cerr << "Error reading " << abs_fname << endl;
+	AutoClosingSAP<ACE_FILE_IO> file;
+	ACE_FILE_Addr addr(file_path.c_str());
+	ACE_FILE_Connector connector;
+	
+	if (connector.connect(file, addr, 0, ACE_Addr::sap_any, 0, O_RDONLY) == -1) {
+		if (log_errors) {
+			std::cerr << "Could not open " << file_path << std::endl;
+		}
 		return false;
 	}
 	
-	ConfigErrorHandler eh(fname);
-	target_structure.load(ostrm.str(), target, eh);
+	std::string content;
 	
+	char buf[4096];
+	ssize_t bytes_read = 0;
+	while ((bytes_read = file.recv(buf, sizeof(buf))) > 0) {
+		content.append(buf, bytes_read);
+	}
+	if (bytes_read != 0) {
+		if (log_errors) {
+			std::cerr << "Error reading " << file_path << std::endl;
+		}
+		return false;
+	}
+	
+	target.swap(content);
+	return true;
+}
+
+/* ====================== Daemon::ConfigLoader =========================*/
+
+Daemon::ConfigLoader::ConfigLoader(std::string const& file_path)
+:	m_filePath(file_path)
+{
+}
+
+Daemon::ConfigLoader::~ConfigLoader()
+{
+}
+
+bool
+Daemon::ConfigLoader::loadAndApply()
+{
+	std::string text;
+	if (!FileOps::readFile(m_filePath, text)) {
+		return false;
+	}
+	
+	Config new_config;
+	ConfigFileStructure new_structure;
+	ConfigErrorHandler eh("config");
+	new_structure.load(text, new_config, eh);
 	if (eh.hasCriticalErrors()) {
 		return false;
 	}
 	
+	GlobalState::WriteAccessor()->swapConfig(new_config);
 	return true;
 }
 
+bool
+Daemon::ConfigLoader::load(
+	Config& new_config, ConfigFileStructure& new_structure)
+{
+	std::string text;
+	if (!FileOps::readFile(m_filePath, text)) {
+		return false;
+	}
+	
+	ConfigErrorHandler eh("config");
+	new_structure.load(text, new_config, eh);
+	return true;
+}
+
+
+/* =================== Daemon::ForwardingConfigLoader ==================*/
+
+Daemon::ForwardingConfigLoader::ForwardingConfigLoader(
+	std::string const& file_path)
+:	m_filePath(file_path)
+{
+}
+
+Daemon::ForwardingConfigLoader::~ForwardingConfigLoader()
+{
+}
+
+bool
+Daemon::ForwardingConfigLoader::loadAndApply(
+	ObsoleteForwardingInfo const& fallback)
+{
+	if (ACE_OS::access(m_filePath.c_str(), F_OK) == -1) {
+		applyFallback(fallback);
+		return true;
+	}
+	
+	std::string text;
+	if (!FileOps::readFile(m_filePath, text)) {
+		return false;
+	}
+	
+	std::istringstream strm(text);
+	ConfigErrorHandler eh("forwarding.xml");
+	
+	Forwarding::Config config;
+	if (!config.fromStream(strm, eh)) {
+		return false;
+	}
+	
+	if (config.options().empty()) {
+		applyFallback(fallback);
+		return true;
+	}
+	
+	Forwarding::Resolver resolver(config);
+	GlobalState::WriteAccessor()->swapForwardingResolver(resolver);
+	return true;
+}
+
+void
+Daemon::ForwardingConfigLoader::applyFallback(
+	ObsoleteForwardingInfo const& fallback)
+{
+	Forwarding::Resolver resolver(fallback.toNewFormat());
+	GlobalState::WriteAccessor()->swapForwardingResolver(resolver);
+}
+
 
 /* ==================== Daemon::UrlPatternsLoader =======================*/
 
-bool
-Daemon::UrlPatternsLoader::load(std::string const& confdir)
+Daemon::UrlPatternsLoader::UrlPatternsLoader(
+	Which which, std::string const& file_path)
+:	m_filePath(file_path),
+	m_which(which)
+{
+}
+
+Daemon::UrlPatternsLoader::~UrlPatternsLoader()
 {
-	if (!load(confdir, "urls", m_combinedPatterns.standardPatterns())) {
-		return false;
-	}
-	if (!load(confdir, "urls.local", m_combinedPatterns.localPatterns())) {
-		return false;
-	}
-	return true;
 }
 
 bool
-Daemon::UrlPatternsLoader::load(
-	std::string const& confdir,
-	std::string const& fname, UrlPatterns& target)
+Daemon::UrlPatternsLoader::loadAndApply()
 {
-	string const abs_fname(confdir+"/"+fname);
-	ifstream istrm(abs_fname.c_str());
-	if (istrm.fail()) {
-		cerr << "Could not open " << abs_fname << endl;
-		return false;
-	}
-	ostringstream ostrm;
-	ostrm << istrm.rdbuf();
-	if (istrm.bad()) {
-		cerr << "Error reading " << abs_fname << endl;
+	std::string text;
+	if (!FileOps::readFile(m_filePath, text)) {
 		return false;
 	}
 	
-	UrlPatternsFile cfile;
-	ConfigErrorHandler eh(fname);
-	cfile.load(ostrm.str(), target, eh);
+	UrlPatterns new_patterns;
+	UrlsFileStructure new_structure;
+	ConfigErrorHandler eh(m_which == URLS ? "urls" : "urls.local");
+	new_structure.load(text, new_patterns, eh);
 	
-	if (eh.hasCriticalErrors()) {
-		return false;
+	GlobalState::WriteAccessor global_state;
+	if (m_which == URLS) {
+		global_state->swapStandardUrlPatterns(new_patterns);
+	} else {
+		global_state->swapLocalUrlPatterns(new_patterns);
 	}
 	
 	return true;
@@ -326,111 +468,98 @@ Daemon::UrlPatternsLoader::load(
 
 /* ====================== Daemon::ContentFilterLoader =========================*/
 
-bool
-Daemon::ContentFiltersLoader::load(std::string const& confdir)
+Daemon::ContentFiltersLoader::ContentFiltersLoader(std::string const& filters_dir)
+:	m_filtersDir(filters_dir)
 {
-	string const dirname(confdir+"/filters");
-	ACE_Dirent dir;
-	if (dir.open(dirname.c_str()) == -1) {
-		cerr << "Could not open directory " << dirname << endl;
-		return false;
+}
+
+Daemon::ContentFiltersLoader::~ContentFiltersLoader()
+{
+}
+
+void
+Daemon::ContentFiltersLoader::loadAndApply()
+{
+	std::vector<std::string> fnames(readFilterFileNames());
+	std::vector<std::string>::const_iterator it(fnames.begin());
+	std::vector<std::string>::const_iterator const end(fnames.end());
+	for (; it != end; ++it) {
+		loadFilter(*it);
 	}
 	
-	deque<string> fnames;
+	AvailableFiltersOrdered ordered_filters(m_filters);
+	GlobalState::WriteAccessor()->swapContentFilters(ordered_filters);
+}
+
+void
+Daemon::ContentFiltersLoader::loadFilter(std::string const& fname)
+{
+	std::string const filter_file_path(m_filtersDir+'/'+fname);
+	std::string const enabled_file_path(filter_file_path+".enabled");
 	
-	for (dirent* ent; (ent = dir.read()); ) {
-		string const fname(ent->d_name);
-		if (fname.find('.') != string::npos) {
+	std::string text;
+	if (!FileOps::readFile(filter_file_path, text)) {
+		return;
+	}
+	
+	FilterFileStructure new_structure;
+	ContentFilterList new_filters;
+	ConfigErrorHandler eh(fname);
+	FilterGroupTag group_tag(fname);
+	new_structure.load(text, new_filters, eh, group_tag);
+	loadEnabledFilterList(new_filters, enabled_file_path);
+	
+	m_filters.add(new_filters);
+}
+
+std::vector<std::string>
+Daemon::ContentFiltersLoader::readFilterFileNames()
+{
+	std::vector<std::string> fnames;
+	
+	ACE_Dirent dir_h;
+	if (dir_h.open(m_filtersDir.c_str()) == -1) {
+		std::cerr << "Could not open directory " << m_filtersDir << std::endl;
+		return fnames;
+	}
+	
+	for (dirent* ent; (ent = dir_h.read()); ) {
+		std::string const fname(ent->d_name);
+		if (fname.find('.') != std::string::npos) {
 			// skip files with extensions
 			continue;
 		}
 		fnames.push_back(fname);
 	}
-	std::sort(fnames.begin(), fnames.end());
 	
-	bool critical_errors = false;	
-	
-	for (; !fnames.empty(); fnames.pop_front()) {
-		string const fname = fnames.front();
-		string const abs_fname(dirname+"/"+fname);
-		ifstream istrm(abs_fname.c_str());
-		if (istrm.fail()) {
-			cerr << "Could not open " << abs_fname << endl;
-			critical_errors = true;
-			continue;
-		}
-		ostringstream ostrm;
-		ostrm << istrm.rdbuf();
-		if (istrm.bad()) {
-			cerr << "Error reading " << abs_fname << endl;
-			critical_errors = true;
-			continue;
-		}
-		
-		ContentFilters filters;
-		ContentFiltersFile cfile;
-		ConfigErrorHandler eh(fname);
-		cfile.load(ostrm.str(), filters, eh, FilterGroupTag());
-		
-		if (eh.hasCriticalErrors()) {
-			critical_errors = true;
-			continue;
-		}
-		
-		loadEnabledFilterList(filters, abs_fname+".enabled");
-		appendEnabledFilters(filters);
-	}
-	
-	m_filters.sortByOrder();
-	return !critical_errors;
+	return fnames;
 }
 
 void
 Daemon::ContentFiltersLoader::loadEnabledFilterList(
-	ContentFilters& filters, std::string const& abs_fname)
+	ContentFilterList& filters, std::string const& enabled_file_path)
 {
-	ifstream istrm(abs_fname.c_str());
-	if (istrm.fail()) {
-		return;
-	}
-	ostringstream ostrm;
-	ostrm << istrm.rdbuf();
-	if (istrm.bad()) {
+	std::string text;
+	if (!FileOps::readFile(enabled_file_path, text, false)) {
 		return;
 	}
 	
 	typedef boost::tokenizer<boost::char_separator<char> > Tokenizer;
 	boost::char_separator<char> sep("\r\n");
-	Tokenizer tokens(ostrm.str(), sep);
-	set<string> enabled_set(tokens.begin(), tokens.end());
+	Tokenizer tokens(text, sep);
+	std::set<std::string> enabled_set(tokens.begin(), tokens.end());
 	bool const all_enabled = (enabled_set.find("*") != enabled_set.end());
-	
-	typedef ContentFilters::FilterList FilterList;
-	FilterList& list = filters.filters();
-	FilterList::iterator it = list.begin();
-	FilterList::iterator const end = list.end();
+
+	ContentFilterList::iterator it(filters.begin());
+	ContentFilterList::iterator const end(filters.end());
 	for (; it != end; ++it) {
-		RegexFilterDescriptor& filter = **it;
+		RegexFilterDescriptor& filter = *it;
 		bool const enabled = all_enabled ||
 			(enabled_set.find(filter.name()) != enabled_set.end());
 		filter.setEnabled(enabled);
 	}
 }
 
-void
-Daemon::ContentFiltersLoader::appendEnabledFilters(ContentFilters const& filters)
-{
-	typedef ContentFilters::FilterList FilterList;
-	FilterList const& list = filters.filters();
-	FilterList::const_iterator it = list.begin();
-	FilterList::const_iterator const end = list.end();
-	for (; it != end; ++it) {
-		if ((*it)->isEnabled()) {
-			m_filters.filters().push_back(*it);
-		}
-	}
-}
-
 
 /* ====================== Daemon::Reconfigurer =========================*/
 
@@ -495,8 +624,8 @@ Daemon::Reconfigurer::reconfigure()
 bool
 Daemon::Reconfigurer::loadConfig()
 {
-	ConfigLoader loader;
-	return loader.load(m_confDir, m_config, m_configFile);
+	ConfigLoader loader(m_confDir+"/config");
+	return loader.load(m_config, m_configFile);
 }
 
 SymbolicInetAddr
@@ -505,7 +634,7 @@ Daemon::Reconfigurer::checkAndFixListenAddr(SymbolicInetAddr const& addr)
 	std::cout << "Checking listen address" << std::endl;
 	
 	SymbolicInetAddr new_addr(addr);
-	if (new_addr.getPort() <= 0 || new_addr.getPort() >= 1<<16) {
+	if (new_addr.getPort() <= 0 || new_addr.getPort() >= (1<<16)) {
 		new_addr = m_defaultListenAddr;
 	}
 	
@@ -597,15 +726,15 @@ Daemon::Reconfigurer::writeListenAddrToConfig(SymbolicInetAddr const& addr)
 	m_config.setListenAddresses(addrs);
 	m_configFile.updateWith(m_config);
 	
-	std::string fname = m_confDir+"/config";
-	ofstream strm(fname.c_str());
+	std::string const fname(m_confDir+"/config");
+	std::ofstream strm(fname.c_str());
 	if (strm.fail()) {
-		std::cout << "Could not open "+fname+" for writing." << std::endl;
+		std::cout << "Could not open " << fname << " for writing." << std::endl;
 		return false;
 	}
 	m_configFile.toStream(strm);
 	if (strm.bad()) {
-		std::cout << "Error writing to "+fname << std::endl;
+		std::cout << "Error writing to " << fname << std::endl;
 		return false;
 	}
 	
@@ -691,7 +820,7 @@ Daemon::run(bool nodaemon, ACE_FILE_IO& pid_file)
 	ProxyWatcherThread proxy_wather;
 #endif
 	
-	m_workerPool.start();
+	m_workerPool.activate();
 	
 	while (!m_sSigtermReceived && !m_sSigintReceived) {
 		sleep(10000);
@@ -824,30 +953,41 @@ Daemon::normalizePath(std::string const& path)
 bool
 Daemon::loadConfiguration(std::string const& confdir)
 {
-	ConfigLoader config_loader;
-	Config config;
-	
 	{
-		ConfigFile config_file;
-		if (!config_loader.load(confdir, config, config_file)) {
+		ConfigLoader config_loader(confdir+"/config");
+		if (!config_loader.loadAndApply()) {
 			return false;
 		}
 	}
 	
-	UrlPatternsLoader patterns_loader;
-	if (!patterns_loader.load(confdir)) {
-		return false;
+	{
+		// !!! assumes config is already loaded !!!
+		ObsoleteForwardingInfo const fwd_fallback(
+			GlobalState::ReadAccessor()->config()
+			.getObsoleteForwardingInfo()
+		);
+		ForwardingConfigLoader fwd_loader(confdir+"/forwarding.xml");
+		if (!fwd_loader.loadAndApply(fwd_fallback)) {
+			return false;
+		}
 	}
 	
-	ContentFiltersLoader cf_loader;
-	if (!cf_loader.load(confdir)) {
-		return false;
+	{
+		UrlPatternsLoader urls_loader(
+			UrlPatternsLoader::URLS, confdir+"/urls");
+		urls_loader.loadAndApply();
 	}
 	
-	GlobalState::WriteAccessor global_state;
-	global_state->config() = config;
-	global_state->urlPatterns() = patterns_loader.getCombinedPatterns();
-	global_state->contentFilters() = cf_loader.getFilters();
+	{
+		UrlPatternsLoader urls_local_loader(
+			UrlPatternsLoader::URLS_LOCAL, confdir+"/urls.local");
+		urls_local_loader.loadAndApply();
+	}
+	
+	{
+		ContentFiltersLoader filters_loader(confdir+"/filters");
+		filters_loader.loadAndApply();
+	}
 	
 	return true;
 }
@@ -859,7 +999,7 @@ Daemon::bind()
 	typedef IntrusivePtr<Acceptor> AcceptorPtr;
 	
 	bool ok = true;
-	deque<AcceptorPtr> acceptors;
+	vector<AcceptorPtr> acceptors;
 	list<SymbolicInetAddr> addrs(GlobalState::ReadAccessor()->config().getListenAddresses());
 	for (; !addrs.empty(); addrs.pop_front()) {
 		SymbolicInetAddr const& addr = addrs.front();
@@ -885,8 +1025,10 @@ Daemon::bind()
 		return false;
 	}
 	
-	for (; !acceptors.empty(); acceptors.pop_front()) {
-		m_workerPool.addAcceptor(acceptors.front());
+	vector<AcceptorPtr>::const_iterator it(acceptors.begin());
+	vector<AcceptorPtr>::const_iterator const end(acceptors.end());
+	for (; it != end; ++it) {
+		m_workerPool.addAcceptor(*it);
 	}
 	
 	return true;
@@ -910,6 +1052,13 @@ Daemon::sigintHandler(int)
 // ACE #define's main() to include its own initialization
 int main(int argc, char **argv)
 {
+	{
+		// Create references to Color and FilterJsLogger,
+		// to prevent link-time errors.
+		Color color(0, 0, 0);
+		FilterJsLogger::setHandler(0);
+	}
+	
 	bool nodaemon = false;
 	bool kill_running = false;
 	bool reconfigure = false;
