@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,6 +26,9 @@
 #include "AutoScrollingWindow.h"
 #include <gtkmm/adjustment.h>
 
+namespace GtkGUI
+{
+
 AutoScrollingWindow::AutoScrollingWindow()
 :	m_savedState(*get_vadjustment()),
 	m_isAtBottom(true),
@@ -106,3 +109,5 @@ AutoScrollingWindow::SavedState::operator==(Gtk::Adjustment const& adj) const
 	return (m_lower == adj.get_lower() && m_upper == adj.get_upper()
 		&& m_pageSize == adj.get_page_size());
 }
+
+} // namespace GtkGUI
