@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,6 +58,9 @@
 
 using namespace std;
 
+namespace GtkGUI
+{
+
 class DebugConnection::Traffic :
 	public RefCountable<RefCounter<ACE_NULL_SYNCH> >
 {
@@ -832,3 +835,5 @@ DebugConnection::SaveTrafficDialog::saveToFile(std::string const& fname)
 	
 	return true;
 }
+
+} // namespace GtkGUI
