@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -47,6 +47,9 @@
 
 using namespace std;
 
+namespace GtkGUI
+{
+
 class DebugWindow::RequestListModel : public Gtk::ListStore
 {
 	struct RequestColumns;
@@ -490,3 +493,5 @@ DebugWindow::Legend::HttpMessageSymbol::HttpMessageSymbol(
 DebugWindow::Legend::HttpMessageSymbol::~HttpMessageSymbol()
 {
 }
+
+} // namespace GtkGUI
