@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,9 +22,9 @@
 #endif
 
 #include "RequestLogHandler.h"
-#include "Application.h"
 #include "RequestLog.h"
 #include "RequestTag.h"
+#include "EncOps.h"
 #include "DownloadProgress.h"
 #include "ArraySize.h"
 #include "CompiledImages.h"
@@ -46,6 +46,9 @@
 
 using namespace std;
 
+namespace GtkGUI
+{
+
 class RequestLogHandler::ListModel : public Gtk::ListStore
 {
 public:
@@ -407,5 +410,7 @@ RequestLogHandler::ListModel::urlToUtf8(std::string const& url)
 		// URL is in ASCII or UTF-8
 		return str;
 	}
-	return Application::localeToUtf8(url);
+	return EncOps::localeToUtf8(url);
 }
+
+} // namespace GtkGUI
