@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,9 @@
 
 using namespace std;
 
+namespace GtkGUI
+{
+
 class RequestLogWindow::LegendModel : public Gtk::ListStore
 {
 public:
@@ -317,8 +320,6 @@ RequestLogWindow::LegendModel::fillModel()
 
 /*===================== RequestLogWindow::LogView ========================*/
 
-#include <iostream>
-
 bool
 RequestLogWindow::LogView::on_button_press_event(GdkEventButton* evt)
 {
@@ -338,3 +339,5 @@ RequestLogWindow::LogView::on_button_press_event(GdkEventButton* evt)
 	
 	return true; // event has been handled
 }
+
+} // namespace GtkGUI
