@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,9 @@
 #include <deque>
 #include <set>
 
+namespace wxGUI
+{
+
 class LogObserver;
 
 class Log : public sigc::trackable
@@ -180,4 +183,6 @@ Log::appendToLastRecord(Iter begin, Iter end)
 	}
 }
 
+} // namespace wxGUI
+
 #endif
