@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,9 @@
 
 using namespace std;
 
+namespace wxGUI
+{
+
 OperationLog::OperationLog()
 {
 	m_ptrMonospacedStyle.reset(new TextStyle);
@@ -87,3 +90,5 @@ OperationLog::appendJsError(
 	text.push_back(StyledText(str2, m_ptrMonospacedStyle));
 	appendRecord(text.begin(), text.end());
 }
+
+} // namespace wxGUI
