@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,6 +41,9 @@
 
 using namespace std;
 
+namespace wxGUI
+{
+
 class RequestLogWindow::LogView : public MvcListView
 {
 public:
@@ -114,10 +117,10 @@ RequestLogWindow::RequestLogWindow()
 	m_pLegendButton = new wxToggleButton(m_pTopPanel, wxID_HELP, _T("Legend"));
 	legend_vbox->Add(m_pLegendButton, 0, wxALIGN_RIGHT);
 	
+	showLog();
+	
 	panel_vbox->SetSizeHints(m_pTopPanel);
 	top_vbox->SetSizeHints(this);
-
-	showLog();
 }
 
 RequestLogWindow::~RequestLogWindow()
@@ -348,3 +351,5 @@ RequestLogWindow::LogView::onDoubleClick(wxMouseEvent& evt)
 		wxGetApp().getRequestLogHandler().copyUrlAtRow(row);
 	}
 }
+
+} // namespace wxGUI
