@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,30 +24,28 @@
 #include <config.h>
 #endif
 
-#include <ace/config-lite.h>
-#include <wx/wx.h>
-#include <wx/menu.h>
+#include "NonCopyable.h"
 
-class TrayMenu : public wxMenu
+class wxCommandEvent;
+
+namespace wxGUI
 {
+
+class TrayIcon;
+
+class TrayMenu
+{
+	DECLARE_NON_COPYABLE(TrayMenu)
 public:
 	TrayMenu();
-private:
-	void onActive(wxCommandEvent& evt);
-	
-	void onConfigBasic(wxCommandEvent& evt);
-	
-	void onConfigAdvanced(wxCommandEvent& evt);
 
-	void onConfigFilters(wxCommandEvent& evt);
-	
-	void onOperationLog(wxCommandEvent& evt);
-	
-	void onRequestLog(wxCommandEvent& evt);
-	
-	void onQuit(wxCommandEvent& evt);
-	
-	DECLARE_EVENT_TABLE()
+	~TrayMenu();
+
+	void popup(TrayIcon& here);
+private:
+	class Impl;
 };
 
+} // namespace wxGUI
+
 #endif
