@@ -111,7 +111,8 @@ PollReactor::registerHandler(
 	ACE_HANDLE handle, EventHandlerPtr const& handler, IOEvents events)
 {
 	assert(handler);
-	
+	assert(handle != ACE_INVALID_HANDLE);
+
 	MtGuard<ACE_Lock> guard(*m_ptrMutex);
 	
 	size_t const required_capacity = m_ptrHandlers->size() + 1;
@@ -296,6 +297,7 @@ PollReactor::handleEvents()
 	int res = 0;
 	{
 		DemultiplexerGuard guard(*m_ptrMutex, *m_ptrDemuxMutex, m_isInsideDemux);
+		
 		while (true) {
 			res = m_pActiveWaitSet->poll(getPollTimeout(), woken_up);
 			if (woken_up) {
@@ -310,6 +312,9 @@ PollReactor::handleEvents()
 			}
 			break;
 		}
+		
+		// Note: another thread may add/remove handlers while we are in
+		// DemultiplexerGuard destructor.
 	}
 	if (res < 0) {
 		assert(!woken_up);
@@ -426,7 +431,7 @@ void
 PollReactor::continueIODispatching()
 {
 	while (!m_isStopped && m_ioIter.next(*m_pActiveWaitSet)) {
-		assert(m_ioIter.pos < m_pActiveWaitSet->size());
+		assert(m_ioIter.pos < (ptrdiff_t)m_pActiveWaitSet->size());
 		ACE_HANDLE const handle = (*m_pActiveWaitSet)[m_ioIter.pos].fd;
 		
 		typedef HandlerRepository::HandleIdx HandleIdx;
@@ -449,13 +454,22 @@ PollReactor::dispatchIO(Handler const& handler)
 	
 	ACE_HANDLE handle = handler.handle;
 	EventHandlerBase* eh = handler.handler.get();
-	if (m_ioIter.phase == 0 && (handler.events & WRITE)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleWrite);
-	} else if (m_ioIter.phase == 1 && (handler.events & EXCEPT)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleExcept);
-	} else if (m_ioIter.phase == 2 && (handler.events & READ)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleRead);
-	}
+	switch (m_ioIter.phase) {
+		case IODispatchIter::WRITE_PHASE:
+		if (handler.events & WRITE) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleWrite);
+		}
+		break;
+		case IODispatchIter::EXCEPT_PHASE:
+		if (handler.events & EXCEPT) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleExcept);
+		}
+		break;
+		case IODispatchIter::READ_PHASE:
+		if (handler.events & READ) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleRead);
+		}
+	};
 }
 
 void
@@ -493,6 +507,23 @@ short translateToPollEvents(Reactor::IOEvents events)
 
 /*===================== PollReactor::IODispatchIter =====================*/
 
+PollReactor::IODispatchIter::IODispatchIter(unsigned todo)
+:	pos(INITIAL_POS),
+	todo(todo),
+	revents(0),
+	phase(INITIAL_PHASE)
+{
+}
+
+void
+PollReactor::IODispatchIter::reset(unsigned todo)
+{
+	pos = INITIAL_POS;
+	this->todo = todo;
+	revents = 0;
+	phase = INITIAL_PHASE;
+}
+
 bool
 PollReactor::IODispatchIter::next(PollWaitSet const& ws)
 {
@@ -500,43 +531,37 @@ PollReactor::IODispatchIter::next(PollWaitSet const& ws)
 		return false;
 	}
 	
-	assert(pos < ws.size());
-	short revents = ws[pos].revents;
-
-	switch (phase) {
-		case -1:
-		case_m1:
-		phase = 0;
-		if (revents & POLL_WRITE) {
-			break;
-		}
-		// fall through
-		case 0:
-		phase = 1;
-		if (revents & POLL_EXCEPT) {
-			break;
-		}
-		// fall through
-		case 1:
-		phase = 2;
-		if (revents & POLL_READ) {
-			break;
-		}
-		// fall through
-		case 2:
-		phase = 0;
-		++pos;
-		if (revents != 0) {
-			if (--todo == 0) {
-				return false;
+	// indexed by phase
+	static short const event_masks[] = {
+		POLL_WRITE, POLL_EXCEPT, POLL_READ
+	};
+	
+	for (;;) {
+		if (++phase > MAX_PHASE) {
+			phase = MIN_PHASE;
+			++pos;
+			if (revents) {
+				if (--todo == 0) {
+					break;
+				}
 			}
+			if (pos >= (ptrdiff_t)ws.size()) {
+				break;
+			}
+			revents = ws[pos].revents;
+		}
+		if (revents & event_masks[phase]) {
+			return true;
 		}
-		assert(pos < ws.size());
-		revents = ws[pos].revents;
-		goto case_m1;
 	}
 	
-	return true;
+	// Note that the situation where todo > 0 && pos >= ws.size() is
+	// possible.  This can happen if a thread calls unregisterAllHandlers()
+	// while another thread is inside poll().
+	// FYI: unregisterAllHandlers() will do its work when the thread
+	// that was inside poll() is inside DemultiplexerGuard destructor.
+	
+	return false;
 }
 
 
