@@ -155,13 +155,23 @@ private:
 	
 	struct IODispatchIter
 	{
-		size_t pos;
+		ptrdiff_t pos;
 		unsigned todo;
-		signed char phase;
+		short revents;
+		unsigned char phase;
 		
-		IODispatchIter(unsigned todo) : pos(0), todo(todo), phase(-1) {}
+		static ptrdiff_t const INITIAL_POS = -1;
 		
-		void reset(unsigned todo) { pos = 0; this->todo = todo; phase = -1; }
+		static unsigned char const WRITE_PHASE = 0;
+		static unsigned char const EXCEPT_PHASE = 1;
+		static unsigned char const READ_PHASE = 2;
+		static unsigned char const INITIAL_PHASE = 2;
+		static unsigned char const MIN_PHASE = 0;
+		static unsigned char const MAX_PHASE = 2;
+		
+		IODispatchIter(unsigned todo);
+		
+		void reset(unsigned todo);
 
 		bool next(ReactorHelpers::PollWaitSet const& ws);
 	};
