@@ -105,6 +105,7 @@ SelectReactor::registerHandler(
 	ACE_HANDLE handle, EventHandlerPtr const& handler, IOEvents events)
 {
 	assert(handler);
+	assert(handle != ACE_INVALID_HANDLE);
 	
 	MtGuard<ACE_Lock> guard(*m_ptrMutex);
 	if (m_isInsideDemux) {
@@ -151,7 +152,7 @@ SelectReactor::enableEvents(ReactorHandlerId const& id, IOEvents events)
 {
 	assert(id);
 	MtGuard<ACE_Lock> guard(*m_ptrMutex);
-	
+
 	HandlerRepository::iterator const& it = HandlerRepository::iterFromId(id);
 	IOEvents const new_events = it->events | events;
 	m_ptrHandlers->setEvents(it, new_events);
@@ -175,7 +176,7 @@ SelectReactor::setEvents(ReactorHandlerId const& id, IOEvents events)
 {
 	assert(id);
 	MtGuard<ACE_Lock> guard(*m_ptrMutex);
-	
+
 	HandlerRepository::iterator const& it = HandlerRepository::iterFromId(id);
 	m_ptrHandlers->setEvents(it, events);
 	setBits(m_waitSet, it->handle, events);
@@ -290,6 +291,9 @@ SelectReactor::handleEvents()
 	{
 		DemultiplexerGuard guard(*m_ptrMutex, *m_ptrDemuxMutex, m_isInsideDemux);
 		res = select(woken_up, interrupted);
+
+		// Note: another thread may add/remove handlers while we are in
+		// DemultiplexerGuard destructor.
 	}
 	if (res < 0) {
 		assert(!woken_up);
@@ -464,13 +468,22 @@ SelectReactor::dispatchIO(Handler const& handler)
 	
 	ACE_HANDLE handle = handler.handle;
 	EventHandlerBase* eh = handler.handler.get();
-	if (m_ioIter.phase == 0 && (handler.events & WRITE)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleWrite);
-	} else if (m_ioIter.phase == 1 && (handler.events & EXCEPT)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleExcept);
-	} else if (m_ioIter.phase == 2 && (handler.events & READ)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleRead);
-	}
+	switch (m_ioIter.phase) {
+		case IODispatchIter::WRITE_PHASE:
+		if (handler.events & WRITE) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleWrite);
+		}
+		break;
+		case IODispatchIter::EXCEPT_PHASE:
+		if (handler.events & EXCEPT) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleExcept);
+		}
+		break;
+		case IODispatchIter::READ_PHASE:
+		if (handler.events & READ) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleRead);
+		}
+	};
 }
 
 void
@@ -524,7 +537,7 @@ SelectReactor::IODispatchIter::IODispatchIter(HandleSet const& hs)
 	read_iter(hs.read),
 	todo(0),
 	handle(ACE_INVALID_HANDLE),
-	phase(0)
+	phase(INITIAL_PHASE)
 {
 }
 
@@ -536,7 +549,7 @@ SelectReactor::IODispatchIter::reset(unsigned todo)
 	read_iter.reset_state();
 	this->todo = todo;
 	handle = ACE_INVALID_HANDLE;
-	phase = 0;
+	phase = INITIAL_PHASE;
 }
 
 bool
@@ -545,30 +558,27 @@ SelectReactor::IODispatchIter::next()
 	if (todo == 0) {
 		return false;
 	}
+	
+	// indexed by phase
+	ACE_Handle_Set_Iterator* iterators[] = {
+		&write_iter, &except_iter, &read_iter
+	};
 
-	switch (phase) {
-		case 0:
-		handle = write_iter();
+	for (int i = 0; i < 3; ++i) {
+		nextPhase();
+		handle = (*iterators[phase])();
 		if (handle != ACE_INVALID_HANDLE) {
-			break;
+			--todo;
+			return true;
 		}
-		phase = 1;
-		// fall through
-		case 1:
-		handle = except_iter();
-		if (handle != ACE_INVALID_HANDLE) {
-			break;
-		}
-		phase = 2;
-		// fall through
-		case 2:
-		handle = read_iter();
-		break;
 	}
 	
-	if (handle != ACE_INVALID_HANDLE) {
-		--todo;
-	}
-	
-	return true;
+	// No more active handles exist, although todo is greater than zero.
+	// This can happen if a thread calls unregisterAllHandlers()
+	// while another thread is inside select().
+	// FYI: unregisterAllHandlers() will do its work when the thread
+	// that was inside select() is inside DemultiplexerGuard destructor.
+
+	todo = 0;
+	return false;
 }
