@@ -113,7 +113,16 @@ private:
 		ACE_Handle_Set_Iterator read_iter;
 		unsigned todo;
 		ACE_HANDLE handle;
-		signed char phase;
+		unsigned char phase;
+		
+		static unsigned char const WRITE_PHASE = 0;
+		static unsigned char const EXCEPT_PHASE = 1;
+		static unsigned char const READ_PHASE = 2;
+		static unsigned char const INITIAL_PHASE = 2;
+		
+		void nextPhase() {
+			phase = static_cast<unsigned char>(0x00000201 >> (phase << 3));
+		}
 		
 		IODispatchIter(HandleSet const& hs);
 		
