@@ -108,6 +108,8 @@ WFMOReactor::registerHandler(
 	ACE_HANDLE handle, EventHandlerPtr const& handler, IOEvents events)
 {
 	assert(handler);
+	assert(handle != ACE_INVALID_HANDLE);
+
 	MtGuard<ACE_Lock> guard(*m_ptrMutex);
 	
 	if (m_ptrHandlers->size() == MAXIMUM_WAIT_OBJECTS - 1) {
@@ -326,6 +328,9 @@ WFMOReactor::handleEvents()
 	{
 		DemultiplexerGuard guard(*m_ptrMutex, *m_ptrDemuxMutex, m_isInsideDemux);
 		res = WaitForMultipleObjects(ws_size, m_events, FALSE, getDemuxTimeout());
+
+		// Note: another thread may add/remove handlers while we are in
+		// DemultiplexerGuard destructor.
 	}
 	if (res == WAIT_FAILED || (res >= WAIT_ABANDONED_0 && res < WAIT_ABANDONED_0 + ws_size)) {
 		return FAILURE;
@@ -485,13 +490,22 @@ WFMOReactor::dispatchIO(Handler const& handler)
 	
 	ACE_HANDLE handle = handler.handle;
 	EventHandlerBase* eh = handler.handler.get();
-	if (m_ioIter.phase == 0 && (handler.events & WRITE)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleWrite);
-	} else if (m_ioIter.phase == 1 && (handler.events & EXCEPT)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleExcept);
-	} else if (m_ioIter.phase == 2 && (handler.events & READ)) {
-		dispatchIOEvent(handle, eh, &EventHandlerBase::handleRead);
-	}
+	switch (m_ioIter.phase) {
+		case IODispatchIter::WRITE_PHASE:
+		if (handler.events & WRITE) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleWrite);
+		}
+		break;
+		case IODispatchIter::EXCEPT_PHASE:
+		if (handler.events & EXCEPT) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleExcept);
+		}
+		break;
+		case IODispatchIter::READ_PHASE:
+		if (handler.events & READ) {
+			dispatchIOEvent(handle, eh, &EventHandlerBase::handleRead);
+		}
+	};
 }
 
 void
@@ -576,44 +590,50 @@ long translateEventsToWFMO(Reactor::IOEvents events)
 
 /*===================== WFMOReactor::IODispatchIter =====================*/
 
+WFMOReactor::IODispatchIter::IODispatchIter()
+:	pos(INITIAL_POS),
+	end(0),
+	revents(0),
+	phase(INITIAL_PHASE)
+{
+}
+
+void
+WFMOReactor::IODispatchIter::reset(unsigned pos, unsigned end)
+{
+	this->pos = pos - 1;
+	this->end = end;
+	revents = 0;
+	phase = INITIAL_PHASE;
+}
+
 bool
 WFMOReactor::IODispatchIter::next(WFMOReactor& reactor)
 {
+	// pos may be unsigned(-1)
 	if (pos == end) {
 		return false;
 	}
-	assert(pos < end);
 	
-	switch (phase) {
-		case -1:
-		case_m1:
-		phase = 0;
-		revents = reactor.queryEvent(pos);
-		if (revents & WFMO_WRITE) {
-			break;
+	// indexed by phase
+	static long const event_masks[] = {
+		WFMO_WRITE, WFMO_EXCEPT, WFMO_READ
+	};
+	
+	for (;;) {
+		if (++phase > MAX_PHASE) {
+			phase = MIN_PHASE;
+			if (++pos == end) {
+				break;
+			}
+			revents = reactor.queryEvent(pos);
 		}
-		// fall through
-		case 0:
-		phase = 1;
-		if (revents & WFMO_EXCEPT) {
-			break;
+		if (revents & event_masks[phase]) {
+			return true;
 		}
-		// fall through
-		case 1:
-		phase = 2;
-		if (revents & WFMO_READ) {
-			break;
-		}
-		// fall through
-		case 2:
-		phase = 0;
-		if (++pos == end) {
-			return false;
-		}
-		goto case_m1;
 	}
 	
-	return true;
+	return false;
 }
 
 #endif // HAVE_WFMO_REACTOR
