@@ -104,13 +104,20 @@ private:
 		unsigned pos;
 		unsigned end;
 		long revents;
-		signed char phase;
+		unsigned char phase;
 		
-		IODispatchIter() : pos(0), end(0), revents(0), phase(-1) {}
+		static unsigned const INITIAL_POS = static_cast<unsigned>(-1);
+
+		static unsigned char const WRITE_PHASE = 0;
+		static unsigned char const EXCEPT_PHASE = 1;
+		static unsigned char const READ_PHASE = 2;
+		static unsigned char const INITIAL_PHASE = 2;
+		static unsigned char const MIN_PHASE = 0;
+		static unsigned char const MAX_PHASE = 2;
+
+		IODispatchIter();
 		
-		void reset(unsigned pos, unsigned end) {
-			this->pos = pos; this->end = end; revents = 0; phase = -1;
-		}
+		void reset(unsigned pos, unsigned end);
 		
 		bool next(WFMOReactor& reactor);
 	};
