@@ -1,6 +1,6 @@
 /*
     BFilter - a smart ad-filtering web proxy
-    Copyright (C) 2002-2006  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,12 @@
 #include <config.h>
 #endif
 
+#include "IPv4SubnetParserTestSuite.h"
 #include "HtmlLexerTestSuite.h"
+#include "XmlLexerTestSuite.h"
+#include "ForwardingTestSuite.h"
+#include "FilterOrderingTestSuite.h"
+#include "HeuristicScoreTestSuite.h"
 #include <boost/test/unit_test.hpp>
 #include <memory>
 
@@ -31,7 +36,12 @@ test_suite* init_unit_test_suite(int, char*[])
 {
 	test_suite* top_ts = BOOST_TEST_SUITE("Master test suite");
 	
+	top_ts->add(new IPv4SubnetParserTestSuite);
 	top_ts->add(new HtmlLexerTestSuite);
+	top_ts->add(new XmlLexerTestSuite);
+	top_ts->add(new ForwardingTestSuite);
+	top_ts->add(new FilterOrderingTestSuite);
+	top_ts->add(new HeuristicScoreTestSuite);
 	
 	return top_ts;
 }
